@@ -15,12 +15,49 @@
 **
 *******************************************************************************
 **
-** This file contains C functions and procedures used by CGI programs
-** (Fossil launched as CGI) to interpret CGI environment variables,
-** gather the results, and send they reply back to the CGI server.
-** This file also contains routines for running a simple web-server
-** (the "fossil ui" or "fossil server" command) and launching subprocesses
-** to handle each inbound HTTP request using CGI.
+** This file began as a set of C functions and procedures used intepret
+** CGI environment variables for Fossil web pages that were invoked by
+** CGI.  That's where the file name comes from.  But over the years it
+** has grown to incorporate lots of related functionality, including:
+**
+**   *  Interpreting CGI environment variables when Fossil is run as
+**      CGI (the original purpose).
+**
+**   *  Interpreting HTTP requests received directly or via an SSH tunnel.
+**
+**   *  Interpreting SCGI requests
+**
+**   *  Generating appropriate replies to CGI, SCGI, and HTTP requests.
+**
+**   *  Listening for incoming HTTP requests and dispatching them.
+**      (Used by "fossil ui" and "fossil server", for example).
+**
+** So, even though the name of this file implies that it only deals with
+** CGI, in fact, the code in this file is used to interpret webpage requests
+** received by a variety of means, and to generate well-formatted replies
+** to those requests.
+**
+** The code in this file abstracts the web-request so that downstream
+** modules that generate the body of the reply (based on the requested page)
+** do not need to know if the request is coming from CGI, direct HTTP, 
+** SCGI, or some other means.
+**
+** This module gathers information about web page request into a key/value
+** store.  Keys and values come from:
+**
+**    *  Query parameters
+**    *  POST parameter
+**    *  Cookies
+**    *  Environment variables
+**
+** The parameters are accessed using cgi_parameter() and similar functions
+** or their convenience macros P() and similar.
+**
+** Environment variable parameters are set as if the request were coming
+** in over CGI even if the request arrived via SCGI or direct HTTP.  Thus
+** the downstream modules that are trying to interpret the request do not
+** need to know the request protocol - they can just request the values
+** of environment variables and everything will always work.
 **
 ** This file contains routines used by Fossil when it is acting as a
 ** CGI client.  For the code used by Fossil when it is acting as a
@@ -49,6 +86,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
+#include <assert.h>
 #include "cgi.h"
 #include "cygsup.h"
 
@@ -94,13 +132,24 @@
 #endif /* INTERFACE */
 
 /*
-** The HTTP reply is generated in two pieces: the header and the body.
+** The reply content is generated in two pieces: the header and the body.
 ** These pieces are generated separately because they are not necessarily
 ** produced in order.  Parts of the header might be built after all or
 ** part of the body.  The header and body are accumulated in separate
 ** Blob structures then output sequentially once everything has been
 ** built.
 **
+** Do not confuse the content header with the HTTP header. The content header
+** is generated by downstream code.  The HTTP header is generated by the
+** cgi_reply() routine below.
+**
+** The content header and contenty body are *approximately* the <head>
+** element and the <body> elements for HTML replies.  However this is only
+** approximate. The content header also includes parts of <body> that
+** show the banner and menu bar at the top of each page.  Also note that
+** not all replies are HTML, but there can still be separate header and
+** body sections of the content.
+**
 ** The cgi_destination() interface switches between the buffers.
 */
 static Blob cgiContent[2] = { BLOB_INITIALIZER, BLOB_INITIALIZER };
@@ -126,8 +175,8 @@ void cgi_destination(int dest){
 }
 
 /*
-** Check to see if the header contains the zNeedle string.  Return true
-** if it does and false if it does not.
+** Check to see if the content header or body contains the zNeedle string.
+** Return true if it does and false if it does not.
 */
 int cgi_header_contains(const char *zNeedle){
   return strstr(blob_str(&cgiContent[0]), zNeedle)!=0;
@@ -137,14 +186,14 @@ int cgi_body_contains(const char *zNeedle){
 }
 
 /*
-** Append reply content to what already exists.
+** Append new reply content to what already exists.
 */
 void cgi_append_content(const char *zData, int nAmt){
   blob_append(pContent, zData, nAmt);
 }
 
 /*
-** Reset the HTTP reply text to be an empty string.
+** Reset both reply content buffers to be empty.
 */
 void cgi_reset_content(void){
   blob_reset(&cgiContent[0]);
@@ -152,21 +201,23 @@ void cgi_reset_content(void){
 }
 
 /*
-** Return a pointer to the CGI output blob.
+** Return a pointer to Blob that is currently accumulating reply content.
 */
 Blob *cgi_output_blob(void){
   return pContent;
 }
 
 /*
-** Return complete text of the output header
+** Return the content header as a text string
 */
 const char *cgi_header(void){
   return blob_str(&cgiContent[0]);
 }
 
 /*
-** Combine the header and body of the CGI into a single string.
+** Combine the header and body content all into the header buffer.
+** In other words, append the body content to the end of the header
+** content.
 */
 static void cgi_combine_header_and_body(void){
   int size = blob_size(&cgiContent[1]);
@@ -177,7 +228,7 @@ static void cgi_combine_header_and_body(void){
 }
 
 /*
-** Return a pointer to the HTTP reply text.
+** Return a pointer to the combined header+body content.
 */
 char *cgi_extract_content(void){
   cgi_combine_header_and_body();
@@ -195,14 +246,21 @@ static int rangeStart = 0;                   /* Start of Range: */
 static int rangeEnd = 0;                     /* End of Range: plus 1 */
 
 /*
-** Set the reply content type
+** Set the reply content type.
+**
+** The reply content type defaults to "text/html".  It only needs to be
+** changed (by calling this routine) in the exceptional case where some
+** other content type is being returned.
 */
 void cgi_set_content_type(const char *zType){
   zContentType = fossil_strdup(zType);
 }
 
 /*
-** Set the reply content to the specified BLOB.
+** Erase any existing reply content.  Replace is with a pNewContent.
+**
+** This routine erases pNewContent.  In other words, it move pNewContent
+** into the content buffer.
 */
 void cgi_set_content(Blob *pNewContent){
   cgi_reset_content();
@@ -220,7 +278,7 @@ void cgi_set_status(int iStat, const char *zStat){
 }
 
 /*
-** Append text to the header of an HTTP reply
+** Append text to the content header buffer.
 */
 void cgi_append_header(const char *zLine){
   blob_append(&extraHeader, zLine, -1);
@@ -251,7 +309,7 @@ void cgi_set_cookie(
     zPath = g.zTop;
     if( zPath[0]==0 ) zPath = "/";
   }
-  if( g.zBaseURL!=0 && strncmp(g.zBaseURL, "https:", 6)==0 ){
+  if( g.zBaseURL!=0 && fossil_strncmp(g.zBaseURL, "https:", 6)==0 ){
     zSecure = " secure;";
   }
   if( lifetime!=0 ){
@@ -274,15 +332,149 @@ void cgi_set_cookie(
 static int is_gzippable(void){
   if( g.fNoHttpCompress ) return 0;
   if( strstr(PD("HTTP_ACCEPT_ENCODING", ""), "gzip")==0 ) return 0;
-  return strncmp(zContentType, "text/", 5)==0
-    || sqlite3_strglob("application/*xml", zContentType)==0
-    || sqlite3_strglob("application/*javascript", zContentType)==0;
+  /* Maintenance note: this oddball structure is intended to make
+  ** adding new mimetypes to this list less of a performance hit than
+  ** doing a strcmp/glob over a growing set of compressible types. */
+  switch(zContentType ? *zContentType : 0){
+    case (int)'a':
+      if(0==fossil_strncmp("application/",zContentType,12)){
+        const char * z = &zContentType[12];
+        switch(*z){
+          case (int)'j':
+            return fossil_strcmp("javascript", z)==0
+                || fossil_strcmp("json", z)==0;
+          case (int)'w': return fossil_strcmp("wasm", z)==0;
+          case (int)'x':
+            return fossil_strcmp("x-tcl", z)==0
+                || fossil_strcmp("x-tar", z)==0;
+          default:
+            return sqlite3_strglob("*xml", z)==0;
+        }
+      }
+      break;
+    case (int)'i':
+      return fossil_strcmp(zContentType, "image/svg+xml")==0
+        || fossil_strcmp(zContentType, "image/vnd.microsoft.icon")==0;
+    case (int)'t':
+      return fossil_strncmp(zContentType, "text/", 5)==0;
+  }
+  return 0;
+}
+
+
+/*
+** The following routines read or write content from/to the wire for
+** an HTTP request.  Depending on settings the content might be coming
+** from or going to a socket, or a file, or it might come from or go
+** to an SSL decoder/encoder.
+*/
+/*
+** Works like fgets():
+**
+** Read a single line of input into s[].  Ensure that s[] is zero-terminated.
+** The s[] buffer is size bytes and so at most size-1 bytes will be read.
+**
+** Return a pointer to s[] on success, or NULL at end-of-input.
+*/
+static char *cgi_fgets(char *s, int size){
+  if( !g.httpUseSSL ){
+    return fgets(s, size, g.httpIn);
+  }
+#ifdef FOSSIL_ENABLE_SSL
+  return ssl_gets(g.httpSSLConn, s, size);
+#else
+  fossil_fatal("SSL not available");
+#endif
+}
+
+/* Works like fread():
+**
+** Read as many as bytes of content as we can, up to a maximum of nmemb
+** bytes.  Return the number of bytes read.  Return 0 if there is no
+** further input or if an I/O error occurs.
+*/
+size_t cgi_fread(void *ptr, size_t nmemb){
+  if( !g.httpUseSSL ){
+    return fread(ptr, 1, nmemb, g.httpIn);
+  }
+#ifdef FOSSIL_ENABLE_SSL
+  return ssl_read_server(g.httpSSLConn, ptr, nmemb, 1);
+#else
+  fossil_fatal("SSL not available");
+#endif
+}
+
+/* Works like feof():
+**
+** Return true if end-of-input has been reached.
+*/
+int cgi_feof(void){
+  if( !g.httpUseSSL ){
+    return feof(g.httpIn);
+  }
+#ifdef FOSSIL_ENABLE_SSL
+  return ssl_eof(g.httpSSLConn);
+#else
+  return 1;
+#endif
+}
+
+/* Works like fwrite():
+**
+** Try to output nmemb bytes of content.  Return the number of
+** bytes actually written.
+*/
+static size_t cgi_fwrite(void *ptr, size_t nmemb){
+  if( !g.httpUseSSL ){
+    return fwrite(ptr, 1, nmemb, g.httpOut);
+  }
+#ifdef FOSSIL_ENABLE_SSL
+  return ssl_write_server(g.httpSSLConn, ptr, nmemb);
+#else
+  fossil_fatal("SSL not available");
+#endif
+}
+
+/* Works like fflush():
+**
+** Make sure I/O has completed.
+*/
+static void cgi_fflush(void){
+  if( !g.httpUseSSL ){
+    fflush(g.httpOut);
+  }
 }
 
 /*
-** Do a normal HTTP reply
+** Given a Content-Type value, returns a string suitable for appending
+** to the Content-Type header for adding (or not) the "; charset=..."
+** part. It returns an empty string for most types or if zContentType
+** is NULL.
+**
+** See forum post f60dece061c364d1 for the discussions which lead to
+** this. Previously we always appended the charset, but WASM loaders
+** are pedantic and refuse to load any responses which have a
+** charset. Also, adding a charset is not strictly appropriate for
+** most types (and not required for many others which may ostensibly
+** benefit from one, as detailed in that forum post).
+*/
+static const char * content_type_charset(const char *zContentType){
+  if(0==fossil_strncmp(zContentType,"text/",5)){
+    return "; charset=utf-8";
+  }
+  return "";
+}
+
+/*
+** Generate the reply to a web request.  The output might be an
+** full HTTP response, or a CGI response, depending on how things have
+** be set up.
+**
+** The reply consists of a response header (an HTTP or CGI response header)
+** followed by the concatenation of the content header and content body.
 */
 void cgi_reply(void){
+  Blob hdr = BLOB_INITIALIZER;
   int total_size;
   if( iReplyStatus<=0 ){
     iReplyStatus = 200;
@@ -297,35 +489,35 @@ void cgi_reply(void){
       iReplyStatus = 206;
       zReplyStatus = "Partial Content";
     }
-    fprintf(g.httpOut, "HTTP/1.0 %d %s\r\n", iReplyStatus, zReplyStatus);
-    fprintf(g.httpOut, "Date: %s\r\n", cgi_rfc822_datestamp(time(0)));
-    fprintf(g.httpOut, "Connection: close\r\n");
-    fprintf(g.httpOut, "X-UA-Compatible: IE=edge\r\n");
+    blob_appendf(&hdr, "HTTP/1.0 %d %s\r\n", iReplyStatus, zReplyStatus);
+    blob_appendf(&hdr, "Date: %s\r\n", cgi_rfc822_datestamp(time(0)));
+    blob_appendf(&hdr, "Connection: close\r\n");
+    blob_appendf(&hdr, "X-UA-Compatible: IE=edge\r\n");
   }else{
     assert( rangeEnd==0 );
-    fprintf(g.httpOut, "Status: %d %s\r\n", iReplyStatus, zReplyStatus);
+    blob_appendf(&hdr, "Status: %d %s\r\n", iReplyStatus, zReplyStatus);
   }
   if( etag_tag()[0]!=0 ){
-    fprintf(g.httpOut, "ETag: %s\r\n", etag_tag());
-    fprintf(g.httpOut, "Cache-Control: max-age=%d\r\n", etag_maxage());
+    blob_appendf(&hdr, "ETag: %s\r\n", etag_tag());
+    blob_appendf(&hdr, "Cache-Control: max-age=%d\r\n", etag_maxage());
     if( etag_mtime()>0 ){
-      fprintf(g.httpOut, "Last-Modified: %s\r\n",
+      blob_appendf(&hdr, "Last-Modified: %s\r\n",
               cgi_rfc822_datestamp(etag_mtime()));
     }
   }else if( g.isConst ){
     /* isConst means that the reply is guaranteed to be invariant, even
     ** after configuration changes and/or Fossil binary recompiles. */
-    fprintf(g.httpOut, "Cache-Control: max-age=315360000, immutable\r\n");
+    blob_appendf(&hdr, "Cache-Control: max-age=315360000, immutable\r\n");
   }else{
-    fprintf(g.httpOut, "Cache-control: no-cache\r\n");
+    blob_appendf(&hdr, "Cache-control: no-cache\r\n");
   }
 
   if( blob_size(&extraHeader)>0 ){
-    fprintf(g.httpOut, "%s", blob_buffer(&extraHeader));
+    blob_appendf(&hdr, "%s", blob_buffer(&extraHeader));
   }
 
   /* Add headers to turn on useful security options in browsers. */
-  fprintf(g.httpOut, "X-Frame-Options: SAMEORIGIN\r\n");
+  blob_appendf(&hdr, "X-Frame-Options: SAMEORIGIN\r\n");
   /* This stops fossil pages appearing in frames or iframes, preventing
   ** click-jacking attacks on supporting browsers.
   **
@@ -346,7 +538,8 @@ void cgi_reply(void){
   ** the browser, not some shared location.
   */
   if( iReplyStatus!=304 ) {
-    fprintf(g.httpOut, "Content-Type: %s; charset=utf-8\r\n", zContentType);
+    blob_appendf(&hdr, "Content-Type: %s%s\r\n", zContentType,
+                 content_type_charset(zContentType));
     if( fossil_strcmp(zContentType,"application/x-fossil")==0 ){
       cgi_combine_header_and_body();
       blob_compress(&cgiContent[0], &cgiContent[0]);
@@ -361,20 +554,22 @@ void cgi_reply(void){
         blob_reset(&cgiContent[i]);
       }
       gzip_finish(&cgiContent[0]);
-      fprintf(g.httpOut, "Content-Encoding: gzip\r\n");
-      fprintf(g.httpOut, "Vary: Accept-Encoding\r\n");
+      blob_appendf(&hdr, "Content-Encoding: gzip\r\n");
+      blob_appendf(&hdr, "Vary: Accept-Encoding\r\n");
     }
     total_size = blob_size(&cgiContent[0]) + blob_size(&cgiContent[1]);
     if( iReplyStatus==206 ){
-      fprintf(g.httpOut, "Content-Range: bytes %d-%d/%d\r\n",
+      blob_appendf(&hdr, "Content-Range: bytes %d-%d/%d\r\n",
               rangeStart, rangeEnd-1, total_size);
       total_size = rangeEnd - rangeStart; 
     }
-    fprintf(g.httpOut, "Content-Length: %d\r\n", total_size);
+    blob_appendf(&hdr, "Content-Length: %d\r\n", total_size);
   }else{
     total_size = 0;
   }
-  fprintf(g.httpOut, "\r\n");
+  blob_appendf(&hdr, "\r\n");
+  cgi_fwrite(blob_buffer(&hdr), blob_size(&hdr));
+  blob_reset(&hdr);
   if( total_size>0
    && iReplyStatus!=304
    && fossil_strcmp(P("REQUEST_METHOD"),"HEAD")!=0
@@ -389,13 +584,13 @@ void cgi_reply(void){
         if( n>total_size ){
           n = total_size;
         }
-        fwrite(blob_buffer(&cgiContent[i])+rangeStart, 1, n, g.httpOut);
+        cgi_fwrite(blob_buffer(&cgiContent[i])+rangeStart, n);
         rangeStart = 0;
         total_size -= n;
       }
     }
   }
-  fflush(g.httpOut);
+  cgi_fflush();
   CGIDEBUG(("-------- END cgi ---------\n"));
 
   /* After the webpage has been sent, do any useful background
@@ -408,7 +603,8 @@ void cgi_reply(void){
 }
 
 /*
-** Do a redirect request to the URL given in the argument.
+** Generate an HTTP or CGI redirect response that causes a redirect
+** to the URL given in the argument.
 **
 ** The URL must be relative to the base of the fossil server.
 */
@@ -419,7 +615,8 @@ NORETURN void cgi_redirect_with_status(
 ){
   char *zLocation;
   CGIDEBUG(("redirect to %s\n", zURL));
-  if( strncmp(zURL,"http:",5)==0 || strncmp(zURL,"https:",6)==0 ){
+  if( fossil_strncmp(zURL,"http:",5)==0
+      || fossil_strncmp(zURL,"https:",6)==0 ){
     zLocation = mprintf("Location: %s\r\n", zURL);
   }else if( *zURL=='/' ){
     int n1 = (int)strlen(g.zBaseURL);
@@ -473,7 +670,7 @@ void cgi_content_disposition_filename(const char *zFilename){
 
 /*
 ** Return the URL for the caller.  This is obtained from either the
-** referer CGI parameter, if it exists, or the HTTP_REFERER HTTP parameter.
+** "referer" CGI parameter, if it exists, or the HTTP_REFERER HTTP parameter.
 ** If neither exist, return zDefault.
 */
 const char *cgi_referer(const char *zDefault){
@@ -503,14 +700,14 @@ int cgi_csrf_safe(int requirePost){
     if( strcmp(zMethod,"POST")!=0 ) return 0;
   }
   nBase = (int)strlen(g.zBaseURL);
-  if( strncmp(g.zBaseURL,zRef,nBase)!=0 ) return 0;
+  if( fossil_strncmp(g.zBaseURL,zRef,nBase)!=0 ) return 0;
   if( zRef[nBase]!=0 && zRef[nBase]!='/' ) return 0;
   return 1;
 }
 
 /*
-** Information about all query parameters and cookies are stored
-** in these variables.
+** Information about all query parameters, post parameter, cookies and
+** CGI environment variables are stored in a hash table as follows:
 */
 static int nAllocQP = 0; /* Space allocated for aParamQP[] */
 static int nUsedQP = 0;  /* Space actually used in aParamQP[] */
@@ -648,8 +845,8 @@ void cgi_delete_query_parameter(const char *zName){
 }
 
 /*
-** Add a query parameter.  The zName portion is fixed but a copy
-** must be made of zValue.
+** Add an environment varaible value to the parameter set.  The zName
+** portion is fixed but a copy is be made of zValue.
 */
 void cgi_setenv(const char *zName, const char *zValue){
   cgi_set_parameter_nocopy(zName, fossil_strdup(zValue), 0);
@@ -770,7 +967,8 @@ static char *get_bounded_content(
   int nBoundary = strlen(zBoundary);
   *pnContent = len;
   for(i=0; i<len; i++){
-    if( z[i]=='\n' && strncmp(zBoundary, &z[i+1], nBoundary)==0 ){
+    if( z[i]=='\n' && fossil_strncmp(zBoundary, &z[i+1],
+                                     nBoundary)==0 ){
       if( i>0 && z[i-1]=='\r' ) i--;
       z[i] = 0;
       *pnContent = i;
@@ -911,67 +1109,19 @@ static void process_multipart_form_data(char *z, int len){
 
 #ifdef FOSSIL_ENABLE_JSON
 /*
-** Internal helper for cson_data_source_FILE_n().
+** Reads a JSON object from the given blob, which is assumed to have
+** been populated by the caller from stdin, the SSL API, or a file, as
+** appropriate for the particular use case. On success g.json.post is
+** updated to hold the content. On error a FSL_JSON_E_INVALID_REQUEST
+** response is output and fossil_exit() is called (in HTTP mode exit
+** code 0 is used).
 */
-typedef struct CgiPostReadState_ {
-    FILE * fh;
-    unsigned int len;
-    unsigned int pos;
-} CgiPostReadState;
-
-/*
-** cson_data_source_f() impl which reads only up to
-** a specified amount of data from its input FILE.
-** state MUST be a full populated (CgiPostReadState*).
-*/
-static int cson_data_source_FILE_n( void * state,
-                                    void * dest,
-                                    unsigned int * n ){
-    if( ! state || !dest || !n ) return cson_rc.ArgError;
-    else {
-      CgiPostReadState * st = (CgiPostReadState *)state;
-      if( st->pos >= st->len ){
-        *n = 0;
-        return 0;
-      }else if( !*n || ((st->pos + *n) > st->len) ){
-        return cson_rc.RangeError;
-      }else{
-        unsigned int rsz = (unsigned int)fread( dest, 1, *n, st->fh );
-        if( ! rsz ){
-          *n = rsz;
-          return feof(st->fh) ? 0 : cson_rc.IOError;
-        }else{
-          *n = rsz;
-          st->pos += *n;
-          return 0;
-        }
-      }
-    }
-}
-
-/*
-** Reads a JSON object from the first contentLen bytes of zIn.  On
-** g.json.post is updated to hold the content. On error a
-** FSL_JSON_E_INVALID_REQUEST response is output and fossil_exit() is
-** called (in HTTP mode exit code 0 is used).
-**
-** If contentLen is 0 then the whole file is read.
-*/
-void cgi_parse_POST_JSON( FILE * zIn, unsigned int contentLen ){
+void cgi_parse_POST_JSON( Blob * pIn ){
   cson_value * jv = NULL;
-  int rc;
-  CgiPostReadState state;
-  cson_parse_opt popt = cson_parse_opt_empty;
   cson_parse_info pinfo = cson_parse_info_empty;
   assert(g.json.gc.a && "json_bootstrap_early() was not called!");
-  popt.maxDepth = 15;
-  state.fh = zIn;
-  state.len = contentLen;
-  state.pos = 0;
-  rc = cson_parse( &jv,
-                   contentLen ? cson_data_source_FILE_n : cson_data_source_FILE,
-                   contentLen ? (void *)&state : (void *)zIn, &popt, &pinfo );
-  if(rc){
+  jv = cson_parse_Blob(pIn, &pinfo);
+  if( jv==NULL ){
     goto invalidRequest;
   }else{
     json_gc_add( "POST.JSON", jv );
@@ -990,7 +1140,7 @@ void cgi_parse_POST_JSON( FILE * zIn, unsigned int contentLen ){
                            pinfo.line, pinfo.col, pinfo.length,
                            cson_rc_string(pinfo.errorCode));
       json_err( FSL_JSON_E_INVALID_REQUEST, msg, 1 );
-      free(msg);
+      fossil_free(msg);
   }else if(jv && !g.json.post.o){
       json_err( FSL_JSON_E_INVALID_REQUEST,
                 "Request envelope must be a JSON Object (not array).", 1 );
@@ -1036,6 +1186,33 @@ void cgi_trace(const char *z){
 /* Forward declaration */
 static NORETURN void malformed_request(const char *zMsg);
 
+/*
+** Checks the QUERY_STRING environment variable, sets it up
+** via add_param_list() and, if found, applies its "skin"
+** setting. Returns 0 if no QUERY_STRING is set, 1 if it is,
+** and 2 if it sets the skin (in which case the cookie may
+** still need flushing by the page, via cookie_render()).
+*/
+int cgi_setup_query_string(void){
+  int rc = 0;
+  char * z = (char*)P("QUERY_STRING");
+  if( z ){
+    ++rc;
+    z = fossil_strdup(z);
+    add_param_list(z, '&');
+    z = (char*)P("skin");
+    if(z){
+      char *zErr = skin_use_alternative(z, 2);
+      ++rc;
+      if(!zErr && !P("once")){
+        cookie_write_parameter("skin","skin",z);
+      }
+      fossil_free(zErr);
+    }
+  }
+  return rc;
+}
+
 /*
 ** Initialize the query parameter database.  Information is pulled from
 ** the QUERY_STRING environment variable (if it exists), from standard
@@ -1045,6 +1222,10 @@ static NORETURN void malformed_request(const char *zMsg);
 **
 **      REQUEST_URI == SCRIPT_NAME + PATH_INFO
 **
+** Or if QUERY_STRING is not empty:
+**
+**      REQUEST_URI == SCRIPT_NAME + PATH_INFO + '?' + QUERY_STRING
+**
 ** Where "+" means concatenate.  Fossil requires SCRIPT_NAME.  If
 ** REQUEST_URI is provided but PATH_INFO is not, then PATH_INFO is
 ** computed from REQUEST_URI and SCRIPT_NAME.  If PATH_INFO is provided
@@ -1063,14 +1244,16 @@ static NORETURN void malformed_request(const char *zMsg);
 **
 ** CGI Parameter quick reference:
 **
-**                                      REQUEST_URI
-**                               _____________|____________
-**                              /                          \
-**    https://www.fossil-scm.org/forum/info/12736b30c072551a?t=c
-**            \________________/\____/\____________________/ \_/
-**                    |            |             |            |
-**               HTTP_HOST         |        PATH_INFO     QUERY_STRING
-**                            SCRIPT_NAME
+**                                   REQUEST_URI
+**                           _____________|________________
+**                          /                              \
+**    https://fossil-scm.org/forum/info/12736b30c072551a?t=c
+**    \___/   \____________/\____/\____________________/ \_/
+**      |           |          |             |            |
+**      |       HTTP_HOST      |        PATH_INFO     QUERY_STRING
+**      |                      |
+**    REQUEST_SCHEMA         SCRIPT_NAME
+**               
 */
 void cgi_init(void){
   char *z;
@@ -1093,16 +1276,14 @@ void cgi_init(void){
   /* We must have SCRIPT_NAME. If the web server did not supply it, try
   ** to compute it from REQUEST_URI and PATH_INFO. */
   if( zScriptName==0 ){
-    size_t nRU, nPI;
     if( zRequestUri==0 || zPathInfo==0 ){
       malformed_request("missing SCRIPT_NAME");  /* Does not return */
     }
-    nRU = strlen(zRequestUri);
-    nPI = strlen(zPathInfo);
-    if( nRU<nPI ){
-      malformed_request("PATH_INFO is longer than REQUEST_URI");
+    z = strstr(zRequestUri,zPathInfo);
+    if( z==0 ){
+      malformed_request("PATH_INFO not found in REQUEST_URI");
     }
-    zScriptName = fossil_strndup(zRequestUri,(int)(nRU-nPI));
+    zScriptName = fossil_strndup(zRequestUri,(int)(z-zRequestUri));
     cgi_set_parameter("SCRIPT_NAME", zScriptName);
   }
 
@@ -1122,11 +1303,16 @@ void cgi_init(void){
 #endif
   if( zRequestUri==0 ){
     const char *z = zPathInfo;
+    const char *zQS = cgi_parameter("QUERY_STRING",0);
     if( zPathInfo==0 ){
       malformed_request("missing PATH_INFO and/or REQUEST_URI");
     }
     if( z[0]=='/' ) z++;
-    zRequestUri = mprintf("%s/%s", zScriptName, z);
+    if( zQS && zQS[0] ){
+      zRequestUri = mprintf("%s/%s?%s", zScriptName, z, zQS);
+    }else{
+      zRequestUri = mprintf("%s/%s", zScriptName, z);
+    }
     cgi_set_parameter("REQUEST_URI", zRequestUri);
   }
   if( zPathInfo==0 ){
@@ -1167,19 +1353,7 @@ void cgi_init(void){
     }
   }
 
-  z = (char*)P("QUERY_STRING");
-  if( z ){
-    z = fossil_strdup(z);
-    add_param_list(z, '&');
-    z = (char*)P("skin");
-    if(z){
-      char *zErr = skin_use_alternative(z, 2);
-      if(!zErr && !P("once")){
-        cookie_write_parameter("skin","skin",z);
-      }
-      fossil_free(zErr);
-    }
-  }
+  cgi_setup_query_string();
 
   z = (char*)P("REMOTE_ADDR");
   if( z ){
@@ -1197,26 +1371,21 @@ void cgi_init(void){
   }
   blob_zero(&g.cgiIn);
   if( len>0 && zType ){
+    if( blob_read_from_cgi(&g.cgiIn, len)<len ){
+      char *zMsg = mprintf("CGI content-length mismatch:  Wanted %d bytes"
+                           " but got only %d\n", len, blob_size(&g.cgiIn));
+      malformed_request(zMsg);
+    }
     if( fossil_strcmp(zType, "application/x-fossil")==0 ){
-      blob_read_from_channel(&g.cgiIn, g.httpIn, len);
       blob_uncompress(&g.cgiIn, &g.cgiIn);
     }
 #ifdef FOSSIL_ENABLE_JSON
-    else if( noJson==0 && g.json.isJsonMode!=0 
-             && json_can_consume_content_type(zType)!=0 ){
-      cgi_parse_POST_JSON(g.httpIn, (unsigned int)len);
-      /*
-       Potential TODOs:
-
-       1) If parsing fails, immediately return an error response
-       without dispatching the ostensibly-upcoming JSON API.
-      */
+    if( noJson==0 && g.json.isJsonMode!=0
+        && json_can_consume_content_type(zType)!=0 ){
+      cgi_parse_POST_JSON(&g.cgiIn);
       cgi_set_content_type(json_guess_content_type());
     }
 #endif /* FOSSIL_ENABLE_JSON */
-    else{
-      blob_read_from_channel(&g.cgiIn, g.httpIn, len);
-    }
   }
 }
 
@@ -1227,7 +1396,7 @@ void cgi_decode_post_parameters(void){
   int len = blob_size(&g.cgiIn);
   if( len==0 ) return;
   if( fossil_strcmp(g.zContentType,"application/x-www-form-urlencoded")==0
-   || strncmp(g.zContentType,"multipart/form-data",19)==0
+   || fossil_strncmp(g.zContentType,"multipart/form-data",19)==0
   ){
     char *z = blob_str(&g.cgiIn);
     cgi_trace(z);
@@ -1733,7 +1902,7 @@ void cgi_handle_http_request(const char *zIpAddr){
   const char *zScheme = "http";
   char zLine[2000];     /* A single line of input. */
   g.fullHttpReply = 1;
-  if( fgets(zLine, sizeof(zLine),g.httpIn)==0 ){
+  if( cgi_fgets(zLine, sizeof(zLine))==0 ){
     malformed_request("missing HTTP header");
   }
   blob_append(&g.httpHeader, zLine, -1);
@@ -1742,8 +1911,10 @@ void cgi_handle_http_request(const char *zIpAddr){
   if( zToken==0 ){
     malformed_request("malformed HTTP header");
   }
-  if( fossil_strcmp(zToken,"GET")!=0 && fossil_strcmp(zToken,"POST")!=0
-      && fossil_strcmp(zToken,"HEAD")!=0 ){
+  if( fossil_strcmp(zToken,"GET")!=0
+   && fossil_strcmp(zToken,"POST")!=0
+   && fossil_strcmp(zToken,"HEAD")!=0
+  ){
     malformed_request("unsupported HTTP method");
   }
   cgi_setenv("GATEWAY_INTERFACE","CGI/1.0");
@@ -1769,7 +1940,7 @@ void cgi_handle_http_request(const char *zIpAddr){
 
   /* Get all the optional fields that follow the first line.
   */
-  while( fgets(zLine,sizeof(zLine),g.httpIn) ){
+  while( cgi_fgets(zLine,sizeof(zLine)) ){
     char *zFieldName;
     char *zVal;
 
@@ -1848,6 +2019,7 @@ void cgi_handle_ssh_http_request(const char *zIpAddr){
   int i, content_length = 0;
   char zLine[2000];     /* A single line of input. */
 
+  assert( !g.httpUseSSL );
 #ifdef FOSSIL_ENABLE_JSON
   if( nCycles==0 ){ json_bootstrap_early(); }
 #endif
@@ -1989,6 +2161,7 @@ void cgi_handle_ssh_http_request(const char *zIpAddr){
 */
 char *cgi_handle_ssh_probes(char *zLine, int zSize, char *z, char *zToken){
   /* Start looking for probes */
+  assert( !g.httpUseSSL );
   while( fossil_strcmp(zToken, "echo")==0 ){
     zToken = extract_token(z, &z);
     if( zToken==0 ){
@@ -2026,6 +2199,7 @@ void cgi_handle_ssh_transport(const char *zCmd){
   char *z, *zToken;
   char zLine[2000];     /* A single line of input. */
 
+  assert( !g.httpUseSSL );
   /* look for second newline of transport_flip */
   if( fgets(zLine, sizeof(zLine),g.httpIn)==0 ){
     malformed_request("incorrect transport_flip");
@@ -2075,6 +2249,7 @@ void cgi_handle_scgi_request(void){
   int nRead;
   int c, n, m;
 
+  assert( !g.httpUseSSL );
   while( (c = fgetc(g.httpIn))!=EOF && fossil_isdigit((char)c) ){
     nHdr = nHdr*10 + (char)c - '0';
   }
@@ -2106,6 +2281,7 @@ void cgi_handle_scgi_request(void){
 #define HTTP_SERVER_HAD_REPOSITORY 0x0004     /* Was the repository open? */
 #define HTTP_SERVER_HAD_CHECKOUT   0x0008     /* Was a checkout open? */
 #define HTTP_SERVER_REPOLIST       0x0010     /* Allow repo listing */
+#define HTTP_SERVER_NOFORK         0x0020     /* Do not call fork() */
 
 #endif /* INTERFACE */
 
@@ -2181,7 +2357,7 @@ int cgi_http_server(
   }
   if( iPort>mxPort ){
     if( mnPort==mxPort ){
-      fossil_fatal("unable to open listening socket on ports %d", mnPort);
+      fossil_fatal("unable to open listening socket on port %d", mnPort);
     }else{
       fossil_fatal("unable to open listening socket on any"
                    " port in the range %d..%d", mnPort, mxPort);
@@ -2190,14 +2366,15 @@ int cgi_http_server(
   if( iPort>mxPort ) return 1;
   listen(listener,10);
   fossil_print("Listening for %s requests on TCP port %d\n",
-     (flags & HTTP_SERVER_SCGI)!=0?"SCGI":"HTTP",  iPort);
+     (flags & HTTP_SERVER_SCGI)!=0 ? "SCGI" :
+        g.httpUseSSL?"TLS-encrypted HTTPS":"HTTP",  iPort);
   fflush(stdout);
   if( zBrowser ){
     assert( strstr(zBrowser,"%d")!=0 );
     zBrowser = mprintf(zBrowser /*works-like:"%d"*/, iPort);
 #if defined(__CYGWIN__)
     /* On Cygwin, we can do better than "echo" */
-    if( strncmp(zBrowser, "echo ", 5)==0 ){
+    if( fossil_strncmp(zBrowser, "echo ", 5)==0 ){
       wchar_t *wUrl = fossil_utf8_to_unicode(zBrowser+5);
       wUrl[wcslen(wUrl)-2] = 0; /* Strip terminating " &" */
       if( (size_t)ShellExecuteW(0, L"open", wUrl, 0, 0, 1)<33 ){
@@ -2225,7 +2402,11 @@ int cgi_http_server(
       lenaddr = sizeof(inaddr);
       connection = accept(listener, (struct sockaddr*)&inaddr, &lenaddr);
       if( connection>=0 ){
-        child = fork();
+        if( flags & HTTP_SERVER_NOFORK ){
+          child = 0;
+        }else{
+          child = fork();
+        }
         if( child!=0 ){
           if( child>0 ){
             nchildren++;
@@ -2320,11 +2501,21 @@ char *cgi_iso8601_datestamp(void){
     return mprintf("");
   }else{
     return mprintf("%04d-%02d-%02d %02d:%02d:%02d",
-                   pTm->tm_year+1900, pTm->tm_mon, pTm->tm_mday,
+                   pTm->tm_year+1900, pTm->tm_mon+1, pTm->tm_mday,
                    pTm->tm_hour, pTm->tm_min, pTm->tm_sec);
   }
 }
 
+/*
+** COMMAND: test-date
+**
+** Show the current date and time in both RFC822 and ISO8601.
+*/
+void test_date(void){
+  fossil_print("%z = ", cgi_iso8601_datestamp());
+  fossil_print("%z\n", cgi_rfc822_datestamp(time(0)));
+}
+
 /*
 ** Parse an RFC822-formatted timestamp as we'd expect from HTTP and return
 ** a Unix epoch time. <= zero is returned on failure.
@@ -2343,7 +2534,7 @@ time_t cgi_rfc822_parsedate(const char *zDate){
                        &mday, zMonth, &year, &hour, &min, &sec)){
     if( year > 1900 ) year -= 1900;
     for(mon=0; azMonths[mon]; mon++){
-      if( !strncmp( azMonths[mon], zMonth, 3 )){
+      if( !fossil_strncmp( azMonths[mon], zMonth, 3 )){
         int nDay;
         int isLeapYr;
         static int priorDays[] =
