@@ -725,8 +725,7 @@ static void export_config(
 **
 ** Usage: %fossil configuration METHOD ... ?OPTIONS?
 **
-** Where METHOD is one of: export import merge pull push reset.  All methods
-** accept the -R or --repository option to specify a repository.
+** Where METHOD is one of: export import merge pull push reset.
 **
 ** >  fossil configuration export AREA FILENAME
 **
@@ -771,7 +770,7 @@ static void export_config(
 **         the remote repository at URL.
 **
 ** Options:
-**    -R|--repository REPO       Extract info from repository REPO
+**    -R|--repository REPO       Affect repository REPO with changes
 **
 ** See also: [[settings]], [[unset]]
 */
@@ -839,7 +838,7 @@ void configuration_cmd(void){
     if( g.argc==5 ){
       zServer = g.argv[4];
     }
-    url_parse(zServer, URL_PROMPT_PW);
+    url_parse(zServer, URL_PROMPT_PW|URL_USE_CONFIG);
     if( g.url.protocol==0 ) fossil_fatal("no server URL specified");
     user_select();
     url_enable_proxy("via proxy: ");
@@ -847,7 +846,9 @@ void configuration_cmd(void){
     if( strncmp(zMethod, "push", n)==0 ){
       client_sync(0,0,(unsigned)mask,0);
     }else if( strncmp(zMethod, "pull", n)==0 ){
+      if( overwriteFlag ) db_unprotect(PROTECT_USER);
       client_sync(0,(unsigned)mask,0,0);
+      if( overwriteFlag ) db_protect_pop();
     }else{
       client_sync(0,(unsigned)mask,(unsigned)mask,0);
     }
