@@ -987,6 +987,7 @@ static int login_basic_authentication(const char *zIpAddr){
 **    g.perm         Permissions granted to this user
 **    g.anon         Permissions that would be available to anonymous
 **    g.isHuman      True if the user is human, not a spider or robot
+**    g.perm         Populated based on user account's capabilities
 **
 */
 void login_check_credentials(void){
@@ -1180,13 +1181,14 @@ void login_check_credentials(void){
   ** are (potentially) copied to the anonymous permission set; otherwise,
   ** those will be out-of-sync.
   */
-  if( zCap[0]
-   && !g.perm.Hyperlink
-   && g.isHuman
-   && db_get_boolean("auto-hyperlink",1)
-  ){
-    g.perm.Hyperlink = 1;
-    g.javascriptHyperlink = 1;
+  if( zCap[0] && !g.perm.Hyperlink && g.isHuman ){
+    int autoLink = db_get_int("auto-hyperlink",1);
+    if( autoLink==1 ){
+      g.jsHref = 1;
+      g.perm.Hyperlink = 1;
+    }else if( autoLink==2 ){
+      g.perm.Hyperlink = 1;
+    }
   }
 
   /*
@@ -2118,48 +2120,58 @@ void login_group_leave(char **pzErrMsg){
 /*
 ** COMMAND: login-group*
 **
-** Usage: %fossil login-group
-**    or: %fossil login-group join REPO [-name NAME]
-**    or: %fossil login-group leave
+** Usage: %fossil login-group ?SUBCOMMAND? ?OPTIONS?
 **
-** With no arguments, this command shows the login-group to which the
-** repository belongs.
+** Run various subcommands to manage login-group related settings of the open
+** repository or of the repository identified by the -R or --repository option.
 **
-** The "join" command adds this repository to login group to which REPO
-** belongs, or creates a new login group between itself and REPO if REPO
-** does not already belong to a login-group.  When creating a new login-
-** group, the name of the new group is determined by the "--name" option.
+** >  fossil login-group ?-R REPO?
 **
-** The "leave" command takes the repository out of whatever login group
-** it is currently a part of.
+**     Show the login-group to which REPO, or if invoked from within a checkout
+**     the repository on which the current checkout is based, belongs.
+**
+** >  fossil login-group join ?-R? REPO ?--name NAME?
+**
+**     This subcommand must be invoked from within a checkout to either: add
+**     the open repository to the login group that REPO is a member, in which
+**     case the optional "--name" argument is not required; or create a new
+**     login group between the open repository and REPO, in which case the new
+**     group NAME is determined by the mandatory "--name" option. REPO may be
+**     specified with or without the -R flag.
+**
+** >  fossil login-group leave ?-R REPO?
+**
+**     Take the repository REPO, or if invoked from within a checkout the
+**     repository on which the current checkout is based, out of whatever
+**     login group it is a member.
 **
 ** About Login Groups:
 **
 ** A login-group is a set of repositories that share user credentials.
 ** If a user is logged into one member of the group, then that user can
-** access any other group member as long as they have an entry in the
-** USER table of that member.  If a user changes their password using
-** web interface, their password is also automatically changed in every
-** other member of the login group.
+** access any other group member as long as they have an entry in the USER
+** table of that member.  If a user changes their password using web
+** interface, their password is also automatically changed in every other
+** member of the login group.
 */
 void login_group_command(void){
   const char *zLGName;
   const char *zCmd;
   int nCmd;
   Stmt q;
-  db_find_and_open_repository(0,0);
+  db_find_and_open_repository(0, 0);
   if( g.argc>2 ){
     zCmd = g.argv[2];
     nCmd = (int)strlen(zCmd);
     if( strncmp(zCmd,"join",nCmd)==0 && nCmd>=1 ){
       const char *zNewName = find_option("name",0,1);
-      const char *zOther;
+      const char *zOther = g.zRepositoryOption
+        ? g.zRepositoryOption : (g.argc>3 ? g.argv[3] : 0);
       char *zErr = 0;
       verify_all_options();
-      if( g.argc!=4 ){
-        fossil_fatal("unknown extra arguments to \"login-group join\"");
+      if( g.zRepositoryOption ? g.argc!=3 : g.argc!=4 ){
+        fossil_fatal("unexpected argument count for \"login-group join\"");
       }
-      zOther = g.argv[3];
       login_group_leave(&zErr);
       sqlite3_free(zErr);
       zErr = 0;
@@ -2192,7 +2204,7 @@ void login_group_command(void){
     return;
   }
   fossil_print("Now part of login-group \"%s\" with:\n", zLGName);
-  db_prepare(&q, "SELECT value FROM config WHERE name LIKE 'peer-name-%%'");
+  db_prepare(&q, "SELECT value FROM config WHERE name LIKE 'peer-repo-%%'");
   while( db_step(&q)==SQLITE_ROW ){
     fossil_print("  %s\n", db_column_text(&q,0));
   }
