@@ -70,7 +70,7 @@ struct Manifest {
   Blob content;         /* The original content blob */
   int type;             /* Type of artifact.  One of CFTYPE_xxxxx */
   int rid;              /* The blob-id for this manifest */
-  char *zBaseline;      /* Baseline manifest.  The B card. */
+  const char *zBaseline;/* Baseline manifest.  The B card. */
   Manifest *pBaseline;  /* The actual baseline manifest */
   char *zComment;       /* Decoded comment.  The C card. */
   double rDate;         /* Date and time from D card.  0.0 if no D card. */
@@ -132,7 +132,7 @@ static struct {
                      ** but we must limit for historical compatibility */
   /* CFTYPE_CLUSTER    2 */ { "MZ",            "MZ"          },
   /* CFTYPE_CONTROL    3 */ { "DTUZ",          "DTUZ"        },
-  /* CFTYPE_WIKI       4 */ { "DLNPUWZ",       "DLUWZ"       },
+  /* CFTYPE_WIKI       4 */ { "CDLNPUWZ",      "DLUWZ"       },
   /* CFTYPE_TICKET     5 */ { "DJKUZ",         "DJKUZ"       },
   /* CFTYPE_ATTACHMENT 6 */ { "ACDNUZ",        "ADZ"         },
   /* CFTYPE_EVENT      7 */ { "CDENPTUWZ",     "DEWZ"        },
@@ -142,7 +142,7 @@ static struct {
 /*
 ** Names of manifest types
 */
-static const char *azNameOfMType[] = {
+static const char *const azNameOfMType[] = {
   "manifest",
   "cluster",
   "tag",
@@ -170,6 +170,31 @@ static struct {
 */
 static int manifest_crosslink_busy = 0;
 
+/*
+** There are some triggers that need to fire whenever new content
+** is added to the EVENT table, to make corresponding changes to the
+** PENDING_ALERT and CHAT tables.  These are done with TEMP triggers
+** which are created as needed.  The reasons for using TEMP triggers:
+**
+**    *  A small minority of invocations of Fossil need to use those triggers.
+**       So we save CPU cycles in the common case by not having to parse the
+**       trigger definition
+**
+**    *  We don't have to worry about dangling table references inside
+**       of triggers.  For example, we can create a trigger that adds
+**       to the CHAT table.  But an admin can still drop that CHAT table
+**       at any moment, since the trigger that refers to CHAT is a TEMP
+**       trigger and won't persist to cause problems.
+**
+**    *  Because TEMP triggers are defined by the specific version of the
+**       application that is running, we don't have to worry with legacy
+**       compatibility of the triggers.
+**
+** This boolean variable is set when the TEMP triggers for EVENT
+** have been created.
+*/
+static int manifest_event_triggers_are_enabled = 0;
+
 /*
 ** Clear the memory allocated in a manifest object
 */
@@ -291,8 +316,8 @@ static int after_blank_line(const char *z){
 /*
 ** Remove the PGP signature from the artifact, if there is one.
 */
-static void remove_pgp_signature(char **pz, int *pn){
-  char *z = *pz;
+static void remove_pgp_signature(const char **pz, int *pn){
+  const char *z = *pz;
   int n = *pn;
   int i;
   if( strncmp(z, "-----BEGIN PGP SIGNED MESSAGE-----", 34)!=0 ) return;
@@ -320,14 +345,19 @@ static void remove_pgp_signature(char **pz, int *pn){
 **   0123456789 123456789 123456789 123456789
 **   Z aea84f4f863865a8d59d0384e4d2a41c
 */
-static int verify_z_card(const char *z, int n){
+static int verify_z_card(const char *z, int n, Blob *pErr){
+  const char *zHash;
   if( n<35 ) return 0;
   if( z[n-35]!='Z' || z[n-34]!=' ' ) return 0;
   md5sum_init();
   md5sum_step_text(z, n-35);
-  if( memcmp(&z[n-33], md5sum_finish(0), 32)==0 ){
+  zHash = md5sum_finish(0);
+  if( memcmp(&z[n-33], zHash, 32)==0 ){
     return 1;
   }else{
+    if(pErr!=0){
+      blob_appendf(pErr, "incorrect Z-card cksum: expected %.32s", zHash);
+    }
     return 2;
   }
 }
@@ -347,16 +377,15 @@ struct ManifestText {
 ** Return NULL if there are no more tokens on the current line.
 */
 static char *next_token(ManifestText *p, int *pLen){
-  char *z;
   char *zStart;
-  int c;
+  int n;
   if( p->atEol ) return 0;
-  zStart = z = p->z;
-  while( (c=(*z))!=' ' && c!='\n' ){ z++; }
-  *z = 0;
-  p->z = &z[1];
-  p->atEol = c=='\n';
-  if( pLen ) *pLen = z - zStart;
+  zStart = p->z;
+  n = strcspn(p->z, " \n");
+  p->atEol = p->z[n]=='\n';
+  p->z[n] = 0;
+  p->z += n+1;
+  if( pLen ) *pLen = n;
   return zStart;
 }
 
@@ -385,6 +414,19 @@ static char next_card(ManifestText *p){
 */
 #define SYNTAX(T)  {zErr=(T); goto manifest_syntax_error;}
 
+/*
+** A cache of manifest IDs which manifest_parse() has seen in this
+** session.
+*/
+static Bag seenManifests =  Bag_INIT;
+/*
+** Frees all memory owned by the manifest "has-seen" cache.  Intended
+** to be called only from the app's atexit() handler.
+*/
+void manifest_clear_cache(){
+  bag_clear(&seenManifests);
+}
+
 /*
 ** Parse a blob into a Manifest object.  The Manifest object
 ** takes over the input blob and will free it when the
@@ -428,7 +470,6 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
   int isRepeat;
   int nSelfTag = 0;     /* Number of T cards referring to this manifest */
   int nSimpleTag = 0;   /* Number of T cards with "+" prefix */
-  static Bag seen;
   const char *zErr = 0;
   unsigned int m;
   unsigned int seenCard = 0;   /* Which card types have been seen */
@@ -436,11 +477,11 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
 
   if( rid==0 ){
     isRepeat = 1;
-  }else if( bag_find(&seen, rid) ){
+  }else if( bag_find(&seenManifests, rid) ){
     isRepeat = 1;
   }else{
     isRepeat = 0;
-    bag_insert(&seen, rid);
+    bag_insert(&seenManifests, rid);
   }
 
   /* Every structural artifact ends with a '\n' character.  Exit early
@@ -451,29 +492,41 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
   n = blob_size(pContent);
   if( n<=0 || z[n-1]!='\n' ){
     blob_reset(pContent);
-    blob_appendf(pErr, "%s", n ? "not terminated with \\n" : "zero-length");
+    if(pErr!=0){
+      blob_appendf(pErr, "%s", n ? "not terminated with \\n" : "zero-length");
+    }
     return 0;
   }
 
   /* Strip off the PGP signature if there is one.
   */
-  remove_pgp_signature(&z, &n);
+  remove_pgp_signature((const char**)&z, &n);
 
   /* Verify that the first few characters of the artifact look like
   ** a control artifact.
   */
   if( n<10 || z[0]<'A' || z[0]>'Z' || z[1]!=' ' ){
     blob_reset(pContent);
-    blob_appendf(pErr, "line 1 not recognized");
+    if(pErr!=0){
+      blob_appendf(pErr, "line 1 not recognized");
+    }
     return 0;
   }
   /* Then verify the Z-card.
   */
-  if( verify_z_card(z, n)==2 ){
+#if 1
+  /* Disable this ***ONLY*** (ONLY!) when testing hand-written inputs
+     for card-related syntax errors. */
+  if( verify_z_card(z, n, pErr)==2 ){
     blob_reset(pContent);
-    blob_appendf(pErr, "incorrect Z-card cksum");
     return 0;
   }
+#else
+#warning ACHTUNG - z-card check is disabled for testing purposes.
+  if(0 && verify_z_card(NULL, 0, NULL)){
+    /*avoid unused static func error*/
+  }
+#endif
 
   /* Allocate a Manifest object to hold the parsed control artifact.
   */
@@ -489,10 +542,23 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
   x.z = z;
   x.zEnd = &z[n];
   x.atEol = 1;
-  while( (cType = next_card(&x))!=0 && cType>=cPrevType ){
+  while( (cType = next_card(&x))!=0 ){
+    if( cType<cPrevType ){
+      /* Cards must be in increasing order.  However, out-of-order detection
+      ** was broken prior to 2021-02-10 due to a bug.  Furthermore, there
+      ** was a bug in technote generation (prior to 2021-02-10) that caused
+      ** the P card to occur before the N card.  Hence, for historical
+      ** compatibility, we do allow the N card of a technote to occur after
+      ** the P card.  See tickets 15d04de574383d61 and 5e67a7f4041a36ad.
+      */
+      if( cType!='N' || cPrevType!='P' || p->zEventId==0 ){
+        SYNTAX("cards not in lexicographical order");
+      }
+    }
     lineNo++;
     if( cType<'A' || cType>'Z' ) SYNTAX("bad card type");
     seenCard |= 1 << (cType-'A');
+    cPrevType = cType;
     switch( cType ){
       /*
       **     A <filename> <target> ?<source>?
@@ -511,7 +577,7 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
         if( zName==0 || zTarget==0 ) goto manifest_syntax_error;
         if( p->zAttachName!=0 ) goto manifest_syntax_error;
         defossilize(zName);
-        if( !file_is_simple_pathname(zName, 0) ){
+        if( !file_is_simple_pathname_nonstrict(zName) ){
           SYNTAX("invalid filename on A-card");
         }
         defossilize(zTarget);
@@ -590,6 +656,7 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
         p->rEventDate = db_double(0.0,"SELECT julianday(%Q)", next_token(&x,0));
         if( p->rEventDate<=0.0 ) SYNTAX("malformed date on E-card");
         p->zEventId = next_token(&x, &sz);
+        if( p->zEventId==0 ) SYNTAX("missing hash on E-card");
         if( !hname_validate(p->zEventId, sz) ){
           SYNTAX("malformed hash on E-card");
         }
@@ -609,11 +676,12 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
         zName = next_token(&x,0);
         if( zName==0 ) SYNTAX("missing filename on F-card");
         defossilize(zName);
-        if( !file_is_simple_pathname(zName, 0) ){
+        if( !file_is_simple_pathname_nonstrict(zName) ){
           SYNTAX("F-card filename is not a simple path");
         }
         zUuid = next_token(&x, &sz);
         if( p->zBaseline==0 || zUuid!=0 ){
+          if( zUuid==0 ) SYNTAX("missing hash on F-card");
           if( !hname_validate(zUuid,sz) ){
             SYNTAX("F-card hash invalid");
           }
@@ -622,7 +690,7 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
         zPriorName = next_token(&x,0);
         if( zPriorName ){
           defossilize(zPriorName);
-          if( !file_is_simple_pathname(zPriorName, 0) ){
+          if( !file_is_simple_pathname_nonstrict(zPriorName) ){
             SYNTAX("F-card old filename is not a simple path");
           }
         }
@@ -632,13 +700,20 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
                                     p->nFileAlloc*sizeof(p->aFile[0]) );
         }
         i = p->nFile++;
+        if( i>0 && fossil_strcmp(p->aFile[i-1].zName, zName)>=0 ){
+          SYNTAX("incorrect F-card sort order");
+        }
+        if( file_is_reserved_name(zName,-1) ){
+          /* If reserved names leaked into historical manifests due to
+          ** slack oversight by older versions of Fossil, simply ignore
+          ** those files */
+          p->nFile--;
+          break;
+        }
         p->aFile[i].zName = zName;
         p->aFile[i].zUuid = zUuid;
         p->aFile[i].zPerm = zPerm;
         p->aFile[i].zPrior = zPriorName;
-        if( i>0 && fossil_strcmp(p->aFile[i-1].zName, zName)>=0 ){
-          SYNTAX("incorrect F-card sort order");
-        }
         p->type = CFTYPE_MANIFEST;
         break;
       }
@@ -1053,16 +1128,20 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
 
 manifest_syntax_error:
   {
-    char *zUuid = db_text(0, "SELECT uuid FROM blob WHERE rid=%d", rid);
+    char *zUuid = rid_to_uuid(rid);
     if( zUuid ){
-      blob_appendf(pErr, "artifact [%s] ", zUuid);
+      if(pErr!=0){
+        blob_appendf(pErr, "artifact [%s] ", zUuid);
+      }
       fossil_free(zUuid);
     }
   }
-  if( zErr ){
-    blob_appendf(pErr, "line %d: %s", lineNo, zErr);
-  }else{
-    blob_appendf(pErr, "unknown error on line %d", lineNo);
+  if(pErr!=0){
+    if( zErr ){
+      blob_appendf(pErr, "line %d: %s", lineNo, zErr);
+    }else{
+      blob_appendf(pErr, "unknown error on line %d", lineNo);
+    }
   }
   md5sum_init();
   manifest_destroy(p);
@@ -1114,6 +1193,47 @@ Manifest *manifest_get_by_name(const char *zName, int *pRid){
   return p;
 }
 
+/*
+** The input blob is text that may or may not be a valid Fossil
+** control artifact of some kind.  This routine returns true if
+** the input is a well-formed control artifact and false if it
+** is not.
+**
+** This routine is optimized to return false quickly and with minimal
+** work in the common case where the input is some random file.
+*/
+int manifest_is_well_formed(const char *zIn, int nIn){
+  int i;
+  int iRes;
+  Manifest *pManifest;
+  Blob copy, errmsg;
+  remove_pgp_signature(&zIn, &nIn);
+
+  /* Check to see that the file begins with a "card" */
+  if( nIn<3 ) return 0;
+  if( zIn[0]<'A' || zIn[0]>'M' || zIn[1]!=' ' ) return 0;
+
+  /* Check to see that the first card is followed by one more card */
+  for(i=2; i<nIn && zIn[i]!='\n'; i++){}
+  if( i>=nIn-3 ) return 0;
+  i++;
+  if( !fossil_isupper(zIn[i]) || zIn[i]<zIn[0] || zIn[i+1]!=' ' ) return 0;
+
+  /* The checks above will eliminate most random inputs.  If these
+  ** quick checks pass, then we could be dealing with a well-formed
+  ** control artifact.  Make a copy, and run it through the official
+  ** artifact parser.  This is the slow path, but it is rarely taken.
+  */
+  blob_init(&copy, 0, 0);
+  blob_init(&errmsg, 0, 0);
+  blob_append(&copy, zIn, nIn);
+  pManifest = manifest_parse(&copy, 0, &errmsg);
+  iRes = pManifest!=0;  
+  manifest_destroy(pManifest);
+  blob_reset(&errmsg);
+  return iRes;
+}
+
 /*
 ** COMMAND: test-parse-manifest
 **
@@ -1127,34 +1247,58 @@ void manifest_test_parse_cmd(void){
   Blob b;
   int i;
   int n = 1;
-  db_find_and_open_repository(0,0);
+  int isWF;
+  db_find_and_open_repository(OPEN_SUBSTITUTE|OPEN_OK_NOT_FOUND,0);
   verify_all_options();
   if( g.argc!=3 && g.argc!=4 ){
     usage("FILENAME");
   }
   blob_read_from_file(&b, g.argv[2], ExtFILE);
   if( g.argc>3 ) n = atoi(g.argv[3]);
+  isWF = manifest_is_well_formed(blob_buffer(&b), blob_size(&b));
+  fossil_print("manifest_is_well_formed() reports the input %s\n",
+       isWF ? "is ok" : "contains errors");
   for(i=0; i<n; i++){
     Blob b2;
     Blob err;
     blob_copy(&b2, &b);
     blob_zero(&err);
     p = manifest_parse(&b2, 0, &err);
-    if( p==0 ) fossil_print("ERROR: %s\n", blob_str(&err));
+    if( p==0 ){
+      fossil_print("ERROR: %s\n", blob_str(&err));
+    }else if( i==0 || (n==2 && i==1) ){
+      fossil_print("manifest_parse() worked\n");
+    }else if( i==n-1 ){
+      fossil_print("manifest_parse() worked %d more times\n", n-1);
+    }
+    if( (p==0 && isWF) || (p!=0 && !isWF) ){
+      fossil_print("ERROR: manifest_is_well_formed() and "
+                   "manifest_parse() disagree!\n");
+    }
     blob_reset(&err);
     manifest_destroy(p);
   }
+  blob_reset(&b);
 }
 
 /*
 ** COMMAND: test-parse-all-blobs
 **
-** Usage: %fossil test-parse-all-blobs
+** Usage: %fossil test-parse-all-blobs ?OPTIONS?
 **
 ** Parse all entries in the BLOB table that are believed to be non-data
 ** artifacts and report any errors.  Run this test command on historical
 ** repositories after making any changes to the manifest_parse()
 ** implementation to confirm that the changes did not break anything.
+**
+** Options:
+**
+**   --limit N            Parse no more than N artifacts before stopping
+**   --wellformed         Use all BLOB table entries as input, not just
+**                        those entries that are believed to be valid
+**                        artifacts, and verify that the result the
+**                        manifest_is_well_formed() agrees with the
+**                        result of manifest_parse().
 */
 void manifest_test_parse_all_blobs_cmd(void){
   Manifest *p;
@@ -1162,19 +1306,47 @@ void manifest_test_parse_all_blobs_cmd(void){
   Stmt q;
   int nTest = 0;
   int nErr = 0;
+  int N = 1000000000;
+  int bWellFormed;
+  const char *z;
   db_find_and_open_repository(0, 0);
+  z = find_option("limit", 0, 1);
+  if( z ) N = atoi(z);
+  bWellFormed = find_option("wellformed",0,0)!=0;
   verify_all_options();
-  db_prepare(&q, "SELECT DISTINCT objid FROM EVENT");
-  while( db_step(&q)==SQLITE_ROW ){
+  if( bWellFormed ){
+    db_prepare(&q, "SELECT rid FROM blob ORDER BY rid");
+  }else{
+    db_prepare(&q, "SELECT DISTINCT objid FROM EVENT ORDER BY objid");
+  }
+  while( (N--)>0 && db_step(&q)==SQLITE_ROW ){
     int id = db_column_int(&q,0);
     fossil_print("Checking %d       \r", id);
     nTest++;
     fflush(stdout);
     blob_init(&err, 0, 0);
-    p = manifest_get(id, CFTYPE_ANY, &err);
-    if( p==0 ){
-      fossil_print("%d ERROR: %s\n", id, blob_str(&err));
-      nErr++;
+    if( bWellFormed ){
+      Blob content;
+      int isWF;
+      content_get(id, &content);
+      isWF = manifest_is_well_formed(blob_buffer(&content),blob_size(&content));
+      p = manifest_parse(&content, id, &err);
+      if( isWF && p==0 ){
+        fossil_print("%d ERROR: manifest_is_well_formed() reported true "
+                     "but manifest_parse() reports an error: %s\n",
+                     id, blob_str(&err));
+        nErr++;
+      }else if( !isWF && p!=0 ){
+        fossil_print("%d ERROR: manifest_is_well_formed() reported false "
+                     "but manifest_parse() found nothing wrong.\n", id);
+        nErr++;
+      }
+    }else{            
+      p = manifest_get(id, CFTYPE_ANY, &err);
+      if( p==0 ){
+        fossil_print("%d ERROR: %s\n", id, blob_str(&err));
+        nErr++;
+      }
     }
     blob_reset(&err);
     manifest_destroy(p);
@@ -1309,7 +1481,7 @@ static int filename_to_fnid(const char *zFilename){
 ** Compute an appropriate mlink.mperm integer for the permission string
 ** of a file.
 */
-int manifest_file_mperm(ManifestFile *pFile){
+int manifest_file_mperm(const ManifestFile *pFile){
   int mperm = PERM_REG;
   if( pFile && pFile->zPerm){
     if( strstr(pFile->zPerm,"x")!=0 ){
@@ -1682,6 +1854,13 @@ static void add_mlink(
       if( pmid<=0 ) continue;
       add_mlink(pmid, 0, mid, pChild, 0);
     }
+    for(i=0; i<pChild->nCherrypick; i++){
+      if( pChild->aCherrypick[i].zCPTarget[0]=='+'
+       && (pmid = uuid_to_rid(pChild->aCherrypick[i].zCPTarget+1, 0))>0
+      ){
+        add_mlink(pmid, 0, mid, pChild, 0);
+      }
+    }
   }
 }
 
@@ -1698,12 +1877,13 @@ static int manifest_add_checkin_linkages(
   int rid,                   /* The RID of the check-in */
   Manifest *p,               /* Manifest for this check-in */
   int nParent,               /* Number of parents for this check-in */
-  char **azParent            /* hashes for each parent */
+  char * const * azParent    /* hashes for each parent */
 ){
   int i;
   int parentid = 0;
   char zBaseId[30];    /* Baseline manifest RID for deltas.  "NULL" otherwise */
   Stmt q;
+  int nLink;
 
   if( p->zBaseline ){
      sqlite3_snprintf(sizeof(zBaseId), zBaseId, "%d",
@@ -1720,7 +1900,11 @@ static int manifest_add_checkin_linkages(
     if( i==0 ) parentid = pid;
   }
   add_mlink(parentid, 0, rid, p, 1);
-  if( nParent>1 ){
+  nLink = nParent;
+  for(i=0; i<p->nCherrypick; i++){
+    if( p->aCherrypick[i].zCPTarget[0]=='+' ) nLink++;
+  }
+  if( nLink>1 ){
     /* Change MLINK.PID from 0 to -1 for files that are added by merge. */
     db_multi_exec(
       "UPDATE mlink SET pid=-1"
@@ -1729,7 +1913,7 @@ static int manifest_add_checkin_linkages(
       "   AND fnid IN "
       "  (SELECT fnid FROM mlink WHERE mid=%d GROUP BY fnid"
       "    HAVING count(*)<%d)",
-      rid, rid, nParent
+      rid, rid, nLink
     );
   }
   db_prepare(&q, "SELECT cid, isprim FROM plink WHERE pid=%d", rid);
@@ -1778,11 +1962,7 @@ void manifest_reparent_checkin(int rid, const char *zValue){
     z[j] = 0;
     i += j;
   }
-  if( !db_exists("SELECT 1 FROM plink WHERE cid=%d AND pid=%d",
-                 rid, uuid_to_rid(azParent[0],0))
-  ){
-    p = manifest_get(rid, CFTYPE_MANIFEST, 0);
-  }
+  p = manifest_get(rid, CFTYPE_MANIFEST, 0);
   if( p!=0 ){
     db_multi_exec(
        "DELETE FROM plink WHERE cid=%d;"
@@ -1790,8 +1970,8 @@ void manifest_reparent_checkin(int rid, const char *zValue){
        rid, rid
     );
     manifest_add_checkin_linkages(rid,p,nParent,azParent);
+    manifest_destroy(p);
   }
-  manifest_destroy(p);
 reparent_abort:
   fossil_free(azParent);
   fossil_free(zCopy);
@@ -1808,9 +1988,10 @@ reparent_abort:
 void manifest_crosslink_begin(void){
   assert( manifest_crosslink_busy==0 );
   manifest_crosslink_busy = 1;
+  manifest_create_event_triggers();
   db_begin_transaction();
   db_multi_exec(
-     "CREATE TEMP TABLE pending_tkt(uuid TEXT UNIQUE);"
+     "CREATE TEMP TABLE pending_xlink(id TEXT PRIMARY KEY)WITHOUT ROWID;"
      "CREATE TEMP TABLE time_fudge("
      "  mid INTEGER PRIMARY KEY,"    /* The rid of a manifest */
      "  m1 REAL,"                    /* The timestamp on mid */
@@ -1820,6 +2001,17 @@ void manifest_crosslink_begin(void){
   );
 }
 
+/*
+** Add a new entry to the pending_xlink table.
+*/
+static void add_pending_crosslink(char cType, const char *zId){
+  assert( manifest_crosslink_busy==1 );
+  db_multi_exec(
+    "INSERT OR IGNORE INTO pending_xlink VALUES('%c%q')",
+    cType, zId
+  );
+}
+
 #if INTERFACE
 /* Timestamps might be adjusted slightly to ensure that check-ins appear
 ** on the timeline in chronological order.  This is the maximum amount
@@ -1861,16 +2053,24 @@ int manifest_crosslink_end(int flags){
     manifest_reparent_checkin(rid, zValue);
   }
   db_finalize(&q);
-  db_prepare(&q, "SELECT uuid FROM pending_tkt");
+  db_prepare(&q, "SELECT id FROM pending_xlink");
   while( db_step(&q)==SQLITE_ROW ){
-    const char *zUuid = db_column_text(&q, 0);
-    ticket_rebuild_entry(zUuid);
-    if( permitHooks && rc==TH_OK ){
-      rc = xfer_run_script(zScript, zUuid, 0);
+    const char *zId = db_column_text(&q, 0);
+    char cType;
+    if( zId==0 || zId[0]==0 ) continue;
+    cType = zId[0];
+    zId++;
+    if( cType=='t' ){
+      ticket_rebuild_entry(zId);
+      if( permitHooks && rc==TH_OK ){
+        rc = xfer_run_script(zScript, zId, 0);
+      }
+    }else if( cType=='w' ){
+      backlink_wiki_refresh(zId);
     }
   }
   db_finalize(&q);
-  db_multi_exec("DROP TABLE pending_tkt");
+  db_multi_exec("DROP TABLE pending_xlink");
 
   /* If multiple check-ins happen close together in time, adjust their
   ** times by a few milliseconds to make sure they appear in chronological
@@ -1908,6 +2108,28 @@ int manifest_crosslink_end(int flags){
   return ( rc!=TH_ERROR );
 }
 
+/*
+** Activate EVENT triggers if they do not already exist.
+*/
+void manifest_create_event_triggers(void){
+  if( manifest_event_triggers_are_enabled ){
+    return;  /* Triggers already exists.  No-op. */
+  }
+  alert_create_trigger();
+  manifest_event_triggers_are_enabled = 1;  
+}
+
+/*
+** Disable manifest event triggers.  Drop them if they exist, but mark
+** them has having been created so that they won't be recreated.  This
+** is used during "rebuild" to prevent triggers from firing then.
+*/
+void manifest_disable_event_triggers(void){
+  alert_drop_trigger();
+  manifest_event_triggers_are_enabled = 1;
+}
+
+
 /*
 ** Make an entry in the event table for a ticket change artifact.
 */
@@ -1977,12 +2199,26 @@ void manifest_ticket_event(
         pManifest->zTicketUuid);
   }
   fossil_free(zTitle);
-  db_multi_exec(
-    "REPLACE INTO event(type,tagid,mtime,objid,user,comment,brief)"
-    "VALUES('t',%d,%.17g,%d,%Q,%Q,%Q)",
-    tktTagId, pManifest->rDate, rid, pManifest->zUser,
-    blob_str(&comment), blob_str(&brief)
-  );
+  manifest_create_event_triggers();
+  if( db_exists("SELECT 1 FROM event WHERE type='t' AND objid=%d", rid) ){
+    /* The ticket_rebuild_entry() function redoes all of the event entries
+    ** for a ticket whenever a new event appears.  Be careful to only UPDATE
+    ** existing events, so that they do not get turned into alerts by
+    ** the alert trigger. */
+    db_multi_exec(
+      "UPDATE event SET tagid=%d, mtime=%.17g, user=%Q, comment=%Q, brief=%Q"
+      " WHERE objid=%d",
+      tktTagId, pManifest->rDate, pManifest->zUser,
+      blob_str(&comment), blob_str(&brief), rid
+    );
+  }else{
+    db_multi_exec(
+      "REPLACE INTO event(type,tagid,mtime,objid,user,comment,brief)"
+      "VALUES('t',%d,%.17g,%d,%Q,%Q,%Q)",
+      tktTagId, pManifest->rDate, rid, pManifest->zUser,
+      blob_str(&comment), blob_str(&brief)
+    );
+  }
   blob_reset(&comment);
   blob_reset(&brief);
 }
@@ -2001,17 +2237,18 @@ void manifest_ticket_event(
 ** However, if the manifest is PGP signed then the extra line has to be
 ** inserted before the PGP signature (thus invalidating the signature).
 */
-void sterilize_manifest(Blob *p){
+void sterilize_manifest(Blob *p, int eType){
   char *z, *zOrig;
   int n, nOrig;
   static const char zExtraLine[] =
-      "# Remove this line to create a well-formed manifest.\n";
+      "# Remove this line to create a well-formed Fossil %s.\n";
+  const char *zType = eType==CFTYPE_MANIFEST ? "manifest" : "control artifact";
 
   z = zOrig = blob_materialize(p);
   n = nOrig = blob_size(p);
-  remove_pgp_signature(&z, &n);
+  remove_pgp_signature((const char **)&z, &n);
   if( z==zOrig ){
-    blob_append(p, zExtraLine, -1);
+    blob_appendf(p, zExtraLine/*works-like:"%s"*/, zType);
   }else{
     int iEnd;
     Blob copy;
@@ -2019,7 +2256,7 @@ void sterilize_manifest(Blob *p){
     blob_init(p, 0, 0);
     iEnd = (int)(&z[n] - zOrig);
     blob_append(p, zOrig, iEnd);
-    blob_append(p, zExtraLine, -1);
+    blob_appendf(p, zExtraLine/*works-like:"%s"*/, zType);
     blob_append(p, &zOrig[iEnd], -1);
     blob_zero(&copy);
   }
@@ -2039,9 +2276,37 @@ static int tag_compare(const void *a, const void *b){
   return c;
 }
 
+/*
+** Inserts plink entries for FORUM, WIKI, and TECHNOTE manifests. May
+** assert for other manifest types. If a parent entry exists, it also
+** propagates any tags for that parent. This is a no-op if
+** p->nParent==0.
+*/
+static void manifest_add_fwt_plink(int rid, Manifest *p){
+  int i;
+  int parentId = 0;
+  assert(p->type==CFTYPE_WIKI ||
+         p->type==CFTYPE_FORUM ||
+         p->type==CFTYPE_EVENT);
+  for(i=0; i<p->nParent; ++i){
+    int const pid = uuid_to_rid(p->azParent[i], 1);
+    if(0==i){
+      parentId = pid;
+    }
+    db_multi_exec(
+                  "INSERT OR IGNORE INTO plink"
+                  "(pid, cid, isprim, mtime, baseid)"
+                  "VALUES(%d, %d, %d, %.17g, NULL)",
+                  pid, rid, i==0, p->rDate);
+  }
+  if(parentId){
+    tag_propagate_all(parentId);
+  }
+}
+
 /*
 ** Scan artifact rid/pContent to see if it is a control artifact of
-** any key:
+** any type:
 **
 **      *  Manifest
 **      *  Control
@@ -2050,6 +2315,7 @@ static int tag_compare(const void *a, const void *b){
 **      *  Cluster
 **      *  Attachment
 **      *  Event
+**      *  Forum post
 **
 ** If the input is a control artifact, then make appropriate entries
 ** in the auxiliary tables of the database in order to crosslink the
@@ -2076,13 +2342,15 @@ int manifest_crosslink(int rid, Blob *pContent, int flags){
   if( g.fSqlTrace ){
     fossil_trace("-- manifest_crosslink(%d)\n", rid);
   }
+  manifest_create_event_triggers();
   if( (p = manifest_cache_find(rid))!=0 ){
     blob_reset(pContent);
   }else if( (p = manifest_parse(pContent, rid, 0))==0 ){
     assert( blob_is_reset(pContent) || pContent==0 );
     if( (flags & MC_NO_ERRORS)==0 ){
-      fossil_error(1, "syntax error in manifest [%S]",
-                   db_text(0, "SELECT uuid FROM blob WHERE rid=%d",rid));
+      char * zErrUuid = db_text(0, "SELECT uuid FROM blob WHERE rid=%d",rid);
+      fossil_error(1, "syntax error in manifest [%S]", zErrUuid);
+      fossil_free(zErrUuid);
     }
     return 0;
   }
@@ -2107,11 +2375,23 @@ int manifest_crosslink(int rid, Blob *pContent, int flags){
       zScript = xfer_commit_code();
       zUuid = db_text(0, "SELECT uuid FROM blob WHERE rid=%d", rid);
     }
+    if( p->nCherrypick && db_table_exists("repository","cherrypick") ){
+      int i;
+      for(i=0; i<p->nCherrypick; i++){
+        db_multi_exec(
+          "REPLACE INTO cherrypick(parentid,childid,isExclude)"
+          " SELECT rid, %d, %d FROM blob WHERE uuid=%Q",
+          rid, p->aCherrypick[i].zCPTarget[0]=='-',
+          p->aCherrypick[i].zCPTarget+1
+        );
+      }
+    }
     if( !db_exists("SELECT 1 FROM mlink WHERE mid=%d", rid) ){
       char *zCom;
       parentid = manifest_add_checkin_linkages(rid,p,p->nParent,p->azParent);
       search_doc_touch('c', rid, 0);
-      db_multi_exec(
+      assert( manifest_event_triggers_are_enabled );
+      zCom = db_text(0,
         "REPLACE INTO event(type,mtime,objid,user,comment,"
                            "bgcolor,euser,ecomment,omtime)"
         "VALUES('ci',"
@@ -2122,16 +2402,15 @@ int manifest_crosslink(int rid, Blob *pContent, int flags){
         "  %d,%Q,%Q,"
         "  (SELECT value FROM tagxref WHERE tagid=%d AND rid=%d AND tagtype>0),"
         "  (SELECT value FROM tagxref WHERE tagid=%d AND rid=%d),"
-        "  (SELECT value FROM tagxref WHERE tagid=%d AND rid=%d),%.17g);",
+        "  (SELECT value FROM tagxref WHERE tagid=%d AND rid=%d),%.17g)"
+        "RETURNING coalesce(ecomment,comment);",
         TAG_DATE, rid, p->rDate,
         rid, p->zUser, p->zComment,
         TAG_BGCOLOR, rid,
         TAG_USER, rid,
         TAG_COMMENT, rid, p->rDate
       );
-      zCom = db_text(0, "SELECT coalesce(ecomment, comment) FROM event"
-                        " WHERE rowid=last_insert_rowid()");
-      wiki_extract_links(zCom, rid, 0, p->rDate, 1, WIKI_INLINE);
+      backlink_extract(zCom, MT_NONE, rid, BKLNK_COMMENT, p->rDate, 1);
       fossil_free(zCom);
 
       /* If this is a delta-manifest, record the fact that this repository
@@ -2180,6 +2459,7 @@ int manifest_crosslink(int rid, Blob *pContent, int flags){
           case '*':  type = 2;  break;  /* Propagate to descendants */
           default:
             fossil_error(1, "unknown tag type in manifest: %s", p->aTag);
+            manifest_destroy(p);
             return 0;
         }
         tag_insert(&p->aTag[i].zName[1], type, p->aTag[i].zValue,
@@ -2190,51 +2470,52 @@ int manifest_crosslink(int rid, Blob *pContent, int flags){
       tag_propagate_all(parentid);
     }
   }
+  if(p->type==CFTYPE_WIKI || p->type==CFTYPE_FORUM
+     || p->type==CFTYPE_EVENT){
+    manifest_add_fwt_plink(rid, p);
+  }
   if( p->type==CFTYPE_WIKI ){
     char *zTag = mprintf("wiki-%s", p->zWikiTitle);
-    int tagid = tag_findid(zTag, 1);
-    int prior;
-    char *zComment;
+    int prior = 0;
+    char cPrefix;
     int nWiki;
     char zLength[40];
+
     while( fossil_isspace(p->zWiki[0]) ) p->zWiki++;
     nWiki = strlen(p->zWiki);
     sqlite3_snprintf(sizeof(zLength), zLength, "%d", nWiki);
     tag_insert(zTag, 1, zLength, rid, p->rDate, rid);
     fossil_free(zTag);
-    prior = db_int(0,
-      "SELECT rid FROM tagxref"
-      " WHERE tagid=%d AND mtime<%.17g"
-      " ORDER BY mtime DESC",
-      tagid, p->rDate
-    );
+    if(p->nParent){
+      prior = fast_uuid_to_rid(p->azParent[0]);
+    }
     if( prior ){
       content_deltify(prior, &rid, 1, 0);
     }
-    if( nWiki>0 ){
-      zComment = mprintf("Changes to wiki page [%h]", p->zWikiTitle);
+    if( nWiki<=0 ){
+      cPrefix = '-';
+    }else if( !prior ){
+      cPrefix = '+';
     }else{
-      zComment = mprintf("Deleted wiki page [%h]", p->zWikiTitle);
+      cPrefix = ':';
     }
     search_doc_touch('w',rid,p->zWikiTitle);
+    if( manifest_crosslink_busy ){
+      add_pending_crosslink('w',p->zWikiTitle);
+    }else{
+      backlink_wiki_refresh(p->zWikiTitle);
+    }
+    assert( manifest_event_triggers_are_enabled );
     db_multi_exec(
-      "REPLACE INTO event(type,mtime,objid,user,comment,"
-      "                  bgcolor,euser,ecomment)"
-      "VALUES('w',%.17g,%d,%Q,%Q,"
-      "  (SELECT value FROM tagxref WHERE tagid=%d AND rid=%d AND tagtype>1),"
-      "  (SELECT value FROM tagxref WHERE tagid=%d AND rid=%d),"
-      "  (SELECT value FROM tagxref WHERE tagid=%d AND rid=%d));",
-      p->rDate, rid, p->zUser, zComment,
-      TAG_BGCOLOR, rid,
-      TAG_USER, rid,
-      TAG_COMMENT, rid
+      "REPLACE INTO event(type,mtime,objid,user,comment)"
+      "VALUES('w',%.17g,%d,%Q,'%c%q');",
+      p->rDate, rid, p->zUser, cPrefix, p->zWikiTitle
     );
-    fossil_free(zComment);
   }
   if( p->type==CFTYPE_EVENT ){
     char *zTag = mprintf("event-%s", p->zEventId);
     int tagid = tag_findid(zTag, 1);
-    int prior, subsequent;
+    int prior = 0, subsequent;
     int nWiki;
     char zLength[40];
     Stmt qatt;
@@ -2243,13 +2524,12 @@ int manifest_crosslink(int rid, Blob *pContent, int flags){
     sqlite3_snprintf(sizeof(zLength), zLength, "%d", nWiki);
     tag_insert(zTag, 1, zLength, rid, p->rDate, rid);
     fossil_free(zTag);
-    prior = db_int(0,
-      "SELECT rid FROM tagxref"
-      " WHERE tagid=%d AND mtime<%.17g AND rid!=%d"
-      " ORDER BY mtime DESC",
-      tagid, p->rDate, rid
-    );
+    if(p->nParent){
+      prior = fast_uuid_to_rid(p->azParent[0]);
+    }
     subsequent = db_int(0,
+      /* BUG: this check is only correct if subsequent
+         version has already been crosslinked. */
       "SELECT rid FROM tagxref"
       " WHERE tagid=%d AND mtime>=%.17g AND rid!=%d"
       " ORDER BY mtime",
@@ -2271,6 +2551,7 @@ int manifest_crosslink(int rid, Blob *pContent, int flags){
       content_deltify(rid, &subsequent, 1, 0);
     }else{
       search_doc_touch('e',rid,0);
+      assert( manifest_event_triggers_are_enabled );
       db_multi_exec(
         "REPLACE INTO event(type,mtime,objid,tagid,user,comment,bgcolor)"
         "VALUES('e',%.17g,%d,%d,%Q,%Q,"
@@ -2317,8 +2598,7 @@ int manifest_crosslink(int rid, Blob *pContent, int flags){
     zTag = mprintf("tkt-%s", p->zTicketUuid);
     tag_insert(zTag, 1, 0, rid, p->rDate, rid);
     fossil_free(zTag);
-    db_multi_exec("INSERT OR IGNORE INTO pending_tkt VALUES(%Q)",
-                  p->zTicketUuid);
+    add_pending_crosslink('t',p->zTicketUuid);
     /* Locate and update comment for any attachments */
     db_prepare(&qatt,
        "SELECT attachid, src, target, filename FROM attachment"
@@ -2355,7 +2635,7 @@ int manifest_crosslink(int rid, Blob *pContent, int flags){
     ** process the attachment artifact before the artifact to
     ** which it is attached!) */
     char attachToType = 'w';
-    if( fossil_is_uuid(p->zAttachTarget) ){
+    if( fossil_is_artifact_hash(p->zAttachTarget) ){
       if( db_exists("SELECT 1 FROM tag WHERE tagname='tkt-%q'",
             p->zAttachTarget)
         ){
@@ -2412,6 +2692,7 @@ int manifest_crosslink(int rid, Blob *pContent, int flags){
              p->zAttachName, p->zAttachTarget, p->zAttachTarget);
       }
     }
+    assert( manifest_event_triggers_are_enabled );
     db_multi_exec(
         "REPLACE INTO event(type,mtime,objid,user,comment)"
         "VALUES('%c',%.17g,%d,%Q,%Q)",
@@ -2515,6 +2796,7 @@ int manifest_crosslink(int rid, Blob *pContent, int flags){
     }
     /*blob_appendf(&comment, " &#91;[/info/%S | details]&#93;");*/
     if( blob_size(&comment)==0 ) blob_append(&comment, " ", 1);
+    assert( manifest_event_triggers_are_enabled );
     db_multi_exec(
       "REPLACE INTO event(type,mtime,objid,user,comment)"
       "VALUES('g',%.17g,%d,%Q,%Q)",
@@ -2527,11 +2809,12 @@ int manifest_crosslink(int rid, Blob *pContent, int flags){
     char *zFType;
     char *zTitle;
     schema_forum();
+    search_doc_touch('f', rid, 0);
     froot = p->zThreadRoot ? uuid_to_rid(p->zThreadRoot, 1) : rid;
     fprev = p->nParent ? uuid_to_rid(p->azParent[0],1) : 0;
     firt = p->zInReplyTo ? uuid_to_rid(p->zInReplyTo,1) : 0;
     db_multi_exec(
-      "INSERT INTO forumpost(fpid,froot,fprev,firt,fmtime)"
+      "REPLACE INTO forumpost(fpid,froot,fprev,firt,fmtime)"
       "VALUES(%d,%d,nullif(%d,0),nullif(%d,0),%.17g)",
       p->rid, froot, fprev, firt, p->rDate
     );
@@ -2543,6 +2826,7 @@ int manifest_crosslink(int rid, Blob *pContent, int flags){
         zTitle = "(Deleted)";
       }
       zFType = fprev ? "Edit" : "Post";
+      assert( manifest_event_triggers_are_enabled );
       db_multi_exec(
         "REPLACE INTO event(type,mtime,objid,user,comment)"
         "VALUES('f',%.17g,%d,%Q,'%q: %q')",
@@ -2575,6 +2859,7 @@ int manifest_crosslink(int rid, Blob *pContent, int flags){
       }else{
         zFType = "Reply";
       }
+      assert( manifest_event_triggers_are_enabled );
       db_multi_exec(
         "REPLACE INTO event(type,mtime,objid,user,comment)"
         "VALUES('f',%.17g,%d,%Q,'%q: %q')",
@@ -2582,7 +2867,12 @@ int manifest_crosslink(int rid, Blob *pContent, int flags){
       );
       fossil_free(zTitle);
     }
+    if( p->zWiki[0] ){
+      int mimetype = parse_mimetype(p->zMimetype);
+      backlink_extract(p->zWiki, mimetype, rid, BKLNK_FORUM, p->rDate, 1);
+    }
   }
+
   db_end_transaction(0);
   if( permitHooks ){
     rc = xfer_run_common_script();
