@@ -42,24 +42,22 @@ static struct BuiltinSkin {
   char *zSQL;           /* Filled in at run-time with SQL to insert this skin */
 } aBuiltinSkin[] = {
   { "Default",                           "default",           0 },
-  { "Blitz",                             "blitz",             0 },
-  { "Blitz, No Logo",                    "blitz_no_logo",     0 },
-  { "Bootstrap",                         "bootstrap",         0 },
-  { "Xekri",                             "xekri",             0 },
-  { "Original",                          "original",          0 },
-  { "Enhanced Original",                 "enhanced1",         0 },
-  { "Shadow boxes & Rounded Corners",    "rounded1",          0 },
-  { "Eagle",                             "eagle",             0 },
-  { "Black & White, Menu on Left",       "black_and_white",   0 },
-  { "Plain Gray, No Logo",               "plain_gray",        0 },
-  { "Khaki, No Logo",                    "khaki",             0 },
   { "Ardoise",                           "ardoise",           0 },
+  { "Black & White",                     "black_and_white",   0 },
+  { "Blitz",                             "blitz",             0 },
+  { "Bootstrap",                         "bootstrap",         0 },
+  { "Dark Mode",                         "darkmode",          0 },
+  { "Eagle",                             "eagle",             0 },
+  { "Khaki",                             "khaki",             0 },
+  { "Original",                          "original",          0 },
+  { "Plain Gray",                        "plain_gray",        0 },
+  { "Xekri",                             "xekri",             0 },
 };
 
 /*
 ** A skin consists of five "files" named here:
 */
-static const char *azSkinFile[] = { 
+static const char *const azSkinFile[] = {
   "css", "header", "footer", "details", "js"
 };
 
@@ -75,6 +73,12 @@ static const char *azSkinFile[] = {
 static struct BuiltinSkin *pAltSkin = 0;
 static char *zAltSkinDir = 0;
 static int iDraftSkin = 0;
+/*
+** Used by skin_use_alternative() to store the current skin rank skin
+** so that the /skins page can, if warranted, warn the user that skin
+** changes won't have any effect.
+*/
+static int nSkinRank = 5;
 
 /*
 ** Skin details are a set of key/value pairs that define display
@@ -87,10 +91,14 @@ static struct SkinDetail {
   const char *zName;      /* Name of the detail */
   const char *zValue;     /* Value of the detail */
 } aSkinDetail[] = {
-  { "timeline-arrowheads",        "1"  },
-  { "timeline-circle-nodes",      "0"  },
-  { "timeline-color-graph-lines", "0"  },
-  { "white-foreground",           "0"  },
+  { "pikchr-background",          ""      },
+  { "pikchr-fontscale",           ""      },
+  { "pikchr-foreground",          ""      },
+  { "pikchr-scale",               ""      },
+  { "timeline-arrowheads",        "1"     },
+  { "timeline-circle-nodes",      "0"     },
+  { "timeline-color-graph-lines", "0"     },
+  { "white-foreground",           "0"     },
 };
 
 /*
@@ -104,18 +112,58 @@ static struct SkinDetail {
 ** it is assumed to be a directory on disk that holds override css.txt,
 ** footer.txt, and header.txt.  This mode can be used for interactive
 ** development of new skins.
+**
+** The 2nd parameter is a ranking of how important this alternative
+** skin declaration is, and lower values trump higher ones. If a call
+** to this function passes a higher-valued rank than a previous call,
+** the subsequent call becomes a no-op. Only calls with the same or
+** lower rank (i.e. higher priority) will overwrite a previous
+** setting. This approach is used because the CGI/server-time
+** initialization happens in an order which is incompatible with our
+** preferred ranking, making it otherwise more invasive to tell the
+** internals "the --skin flag ranks higher than a URL parameter" (the
+** former gets initialized before both URL parameters and the /draft
+** path determination).
+**
+** The rankings were initially defined in
+** https://fossil-scm.org/forum/forumpost/caf8c9a8bb
+** and are:
+**
+** 0) A skin name matching the glob draft[1-9] trumps everything else.
+**
+** 1) The --skin flag or skin: CGI config setting.
+**
+** 2) The "skin" display setting cookie or URL argument, in that
+** order. If the "skin" URL argument is provided and refers to a legal
+** skin then that will update the display cookie. If the skin name is
+** illegal it is silently ignored.
+**
+** 3) Skin properties from the CONFIG db table
+**
+** 4) Default skin.
+**
+** As a special case, a NULL or empty name resets zAltSkinDir and
+** pAltSkin to 0 to indicate that the current config-side skin should
+** be used (rank 3, above), then returns 0.
 */
-char *skin_use_alternative(const char *zName){
+char *skin_use_alternative(const char *zName, int rank){
   int i;
   Blob err = BLOB_INITIALIZER;
-  if( strchr(zName, '/')!=0 ){
+  if(rank > nSkinRank) return 0;
+  nSkinRank = rank;
+  if( zName && 1==rank && strchr(zName, '/')!=0 ){
     zAltSkinDir = fossil_strdup(zName);
     return 0;
   }
-  if( sqlite3_strglob("draft[1-9]", zName)==0 ){
+  if( zName && sqlite3_strglob("draft[1-9]", zName)==0 ){
     skin_use_draft(zName[5] - '0');
     return 0;
   }
+  if(!zName || !*zName){
+    pAltSkin = 0;
+    zAltSkinDir = 0;
+    return 0;
+  }
   for(i=0; i<count(aBuiltinSkin); i++){
     if( fossil_strcmp(aBuiltinSkin[i].zLabel, zName)==0 ){
       pAltSkin = &aBuiltinSkin[i];
@@ -137,7 +185,7 @@ char *skin_use_alternative(const char *zName){
 void skin_override(void){
   const char *zSkin = find_option("skin",0,1);
   if( zSkin ){
-    char *zErr = skin_use_alternative(zSkin);
+    char *zErr = skin_use_alternative(zSkin, 1);
     if( zErr ) fossil_fatal("%s", zErr);
   }
 }
@@ -283,6 +331,9 @@ static unsigned int skin_hash(unsigned int h, const char *z){
 ** identifier can be attached to resource URLs to force reloading when
 ** the resources change but allow the resources to be read from cache
 ** as long as they are unchanged.
+**
+** The zResource argument is the name of a CONFIG setting that
+** defines the resource.  Examples:  "css", "logo-image".
 */
 unsigned int skin_id(const char *zResource){
   unsigned int h = 0;
@@ -295,7 +346,9 @@ unsigned int skin_id(const char *zResource){
     h = skin_hash(0, zMTime);
     fossil_free(zMTime);
   }
-  h = skin_hash(h, MANIFEST_UUID);
+
+  /* Change the ID every time Fossil is recompiled */
+  h = skin_hash(h, fossil_exe_id());
   return h;
 }
 
@@ -357,10 +410,12 @@ static char *getSkin(const char *zName){
         fossil_free(zLabel);
       }
     }
+    db_unprotect(PROTECT_CONFIG);
     blob_appendf(&val,
        "REPLACE INTO config(name,value,mtime) VALUES(%Q,%Q,now());\n",
        azSkinFile[i], z
     );
+    db_protect_pop();
   }
   return blob_str(&val);
 }
@@ -379,12 +434,13 @@ static int skinRename(void){
   if( zOldName==0 ) return 0;
   if( zNewName==0 || zNewName[0]==0 || (ex = skinExists(zNewName))!=0 ){
     if( zNewName==0 ) zNewName = zOldName;
+    style_set_current_feature("skins");
     style_header("Rename A Skin");
     if( ex ){
       @ <p><span class="generalError">There is already another skin
       @ named "%h(zNewName)".  Choose a different name.</span></p>
     }
-    @ <form action="%s(g.zTop)/setup_skin_admin" method="post"><div>
+    @ <form action="%R/setup_skin_admin" method="post"><div>
     @ <table border="0"><tr>
     @ <tr><td align="right">Current name:<td align="left"><b>%h(zOldName)</b>
     @ <tr><td align="right">New name:<td align="left">
@@ -396,13 +452,15 @@ static int skinRename(void){
     @ </table>
     login_insert_csrf_secret();
     @ </div></form>
-    style_footer();
+    style_finish_page();
     return 1;
   }
+  db_unprotect(PROTECT_CONFIG);
   db_multi_exec(
     "UPDATE config SET name='skin:%q' WHERE name='skin:%q';",
     zNewName, zOldName
   );
+  db_protect_pop();
   return 0;
 }
 
@@ -419,12 +477,13 @@ static int skinSave(const char *zCurrent){
   }
   if( zNewName==0 || zNewName[0]==0 || (ex = skinExists(zNewName))!=0 ){
     if( zNewName==0 ) zNewName = "";
+    style_set_current_feature("skins");
     style_header("Save Current Skin");
     if( ex ){
       @ <p><span class="generalError">There is already another skin
       @ named "%h(zNewName)".  Choose a different name.</span></p>
     }
-    @ <form action="%s(g.zTop)/setup_skin_admin" method="post"><div>
+    @ <form action="%R/setup_skin_admin" method="post"><div>
     @ <table border="0"><tr>
     @ <tr><td align="right">Name for this skin:<td align="left">
     @ <input type="text" size="35" name="svname" value="%h(zNewName)">
@@ -434,14 +493,16 @@ static int skinSave(const char *zCurrent){
     @ </table>
     login_insert_csrf_secret();
     @ </div></form>
-    style_footer();
+    style_finish_page();
     return 1;
   }
+  db_unprotect(PROTECT_CONFIG);
   db_multi_exec(
     "INSERT OR IGNORE INTO config(name, value, mtime)"
     "VALUES('skin:%q',%Q,now())",
     zNewName, zCurrent
   );
+  db_protect_pop();
   return 0;
 }
 
@@ -461,7 +522,7 @@ void setup_skin_admin(void){
   int once;
 
   login_check_credentials();
-  if( !g.perm.Setup ){
+  if( !g.perm.Admin ){
     login_needed(0);
     return;
   }
@@ -471,11 +532,13 @@ void setup_skin_admin(void){
     aBuiltinSkin[i].zSQL = getSkin(aBuiltinSkin[i].zLabel);
   }
 
+  style_set_current_feature("skins");
+
   if( cgi_csrf_safe(1) ){
     /* Process requests to delete a user-defined skin */
     if( P("del1") && (zName = skinVarName(P("sn"), 1))!=0 ){
       style_header("Confirm Custom Skin Delete");
-      @ <form action="%s(g.zTop)/setup_skin_admin" method="post"><div>
+      @ <form action="%R/setup_skin_admin" method="post"><div>
       @ <p>Deletion of a custom skin is a permanent action that cannot
       @ be undone.  Please confirm that this is what you want to do:</p>
       @ <input type="hidden" name="sn" value="%h(P("sn"))" />
@@ -483,28 +546,32 @@ void setup_skin_admin(void){
       @ <input type="submit" name="cancel" value="Cancel - Do Not Delete" />
       login_insert_csrf_secret();
       @ </div></form>
-      style_footer();
+      style_finish_page();
       db_end_transaction(1);
       return;
     }
     if( P("del2")!=0 && (zName = skinVarName(P("sn"), 1))!=0 ){
+      db_unprotect(PROTECT_CONFIG);
       db_multi_exec("DELETE FROM config WHERE name=%Q", zName);
+      db_protect_pop();
     }
     if( P("draftdel")!=0 ){
       const char *zDraft = P("name");
       if( sqlite3_strglob("draft[1-9]",zDraft)==0 ){
+        db_unprotect(PROTECT_CONFIG);
         db_multi_exec("DELETE FROM config WHERE name GLOB '%q-*'", zDraft);
+        db_protect_pop();
       }
     }
     if( skinRename() || skinSave(zCurrent) ){
       db_end_transaction(0);
       return;
     }
-  
+
     /* The user pressed one of the "Install" buttons. */
     if( P("load") && (z = P("sn"))!=0 && z[0] ){
       int seen = 0;
-  
+
       /* Check to see if the current skin is already saved.  If it is, there
       ** is no need to create a backup */
       zCurrent = getSkin(0);
@@ -518,11 +585,13 @@ void setup_skin_admin(void){
         seen = db_exists("SELECT 1 FROM config WHERE name GLOB 'skin:*'"
                          " AND value=%Q", zCurrent);
         if( !seen ){
+          db_unprotect(PROTECT_CONFIG);
           db_multi_exec(
             "INSERT INTO config(name,value,mtime) VALUES("
             "  strftime('skin:Backup On %%Y-%%m-%%d %%H:%%M:%%S'),"
             "  %Q,now())", zCurrent
           );
+          db_protect_pop();
         }
       }
       seen = 0;
@@ -530,18 +599,22 @@ void setup_skin_admin(void){
         if( fossil_strcmp(aBuiltinSkin[i].zDesc, z)==0 ){
           seen = 1;
           zCurrent = aBuiltinSkin[i].zSQL;
+          db_unprotect(PROTECT_CONFIG);
           db_multi_exec("%s", zCurrent/*safe-for-%s*/);
+          db_protect_pop();
           break;
         }
       }
       if( !seen ){
         zName = skinVarName(z,0);
         zCurrent = db_get(zName, 0);
+        db_unprotect(PROTECT_CONFIG);
         db_multi_exec("%s", zCurrent/*safe-for-%s*/);
+        db_protect_pop();
       }
     }
   }
-  
+
   style_header("Skins");
   if( zErr ){
     @ <p style="color:red">%h(zErr)</p>
@@ -555,7 +628,7 @@ void setup_skin_admin(void){
       @ (Currently In Use)
       seenCurrent = 1;
     }else{
-      @ <form action="%s(g.zTop)/setup_skin_admin" method="post">
+      @ <form action="%R/setup_skin_admin" method="post">
       @ <input type="hidden" name="sn" value="%h(z)" />
       @ <input type="submit" name="load" value="Install" />
       if( pAltSkin==&aBuiltinSkin[i] ){
@@ -581,7 +654,7 @@ void setup_skin_admin(void){
       @ in the CONFIG table:</h2></td></tr>
     }
     @ <tr><td>%d(i).<td>%h(zN)<td>&nbsp;&nbsp;<td>
-    @ <form action="%s(g.zTop)/setup_skin_admin" method="post">
+    @ <form action="%R/setup_skin_admin" method="post">
     if( fossil_strcmp(zV, zCurrent)==0 ){
       @ (Currently In Use)
       seenCurrent = 1;
@@ -599,7 +672,7 @@ void setup_skin_admin(void){
     @ <tr><td colspan=4><h2>Current skin in css/header/footer/details entries \
     @ in the CONFIG table:</h2></td></tr>
     @ <tr><td>%d(i).<td><i>Current</i><td>&nbsp;&nbsp;<td>
-    @ <form action="%s(g.zTop)/setup_skin_admin" method="post">
+    @ <form action="%R/setup_skin_admin" method="post">
     @ <input type="submit" name="save" value="Backup">
     @ </form>
   }
@@ -618,7 +691,7 @@ void setup_skin_admin(void){
       @ in the CONFIG table:</h2></td></tr>
     }
     @ <tr><td>%d(i).<td>%h(zN)<td>&nbsp;&nbsp;<td>
-    @ <form action="%s(g.zTop)/setup_skin_admin" method="post">
+    @ <form action="%R/setup_skin_admin" method="post">
     @ <input type="submit" name="draftdel" value="Delete">
     @ <input type="hidden" name="name" value="%h(zN)">
     @ </form></tr>
@@ -626,7 +699,7 @@ void setup_skin_admin(void){
   db_finalize(&q);
 
   @ </table>
-  style_footer();
+  style_finish_page();
   db_end_transaction(0);
 }
 
@@ -674,26 +747,35 @@ static void skin_emit_skin_selector(
 static const char *skin_file_content(const char *zLabel, const char *zFile){
   const char *zResult;
   if( fossil_strcmp(zLabel, "current")==0 ){
-    zResult = db_get(zFile, "");
+    zResult = skin_get(zFile);
   }else if( sqlite3_strglob("draft[1-9]", zLabel)==0 ){
     zResult = db_get_mprintf("", "%s-%s", zLabel, zFile);
   }else{
-    while( 1 ){
+    int i;
+    for(i=0; i<2; i++){
       char *zKey = mprintf("skins/%s/%s.txt", zLabel, zFile);
       zResult = builtin_text(zKey);
       fossil_free(zKey);
-      if( zResult!=0 || fossil_strcmp(zLabel,"default")==0 ) break;
+      if( zResult!=0 ) break;
+      zLabel = "default";
     }
   }
   return zResult;
 }
 
+extern const struct strctCssDefaults {
+/* From the generated default_css.h, which we cannot #include here
+** without causing an ODR violation.
+*/
+  const char *elementClass;  /* Name of element needed */
+  const char *value;         /* CSS text */
+} cssDefaultList[];
 
 /*
 ** WEBPAGE: setup_skinedit
 **
 ** Edit aspects of a skin determined by the w= query parameter.
-** Requires Setup privileges.
+** Requires Admin or Setup privileges.
 **
 **    w=NUM     -- 0=CSS, 1=footer, 2=header, 3=details, 4=js
 **    sk=NUM    -- the draft skin number
@@ -715,7 +797,6 @@ void setup_skinedit(void){
   const char *zContent;       /* Content after editing */
   const char *zDflt;          /* Default content */
   char *zDraft;               /* Which draft:  "draft%d" */
-  char *zKey;                 /* CONFIG table key name: "draft%d-%s" */
   char *zTitle;               /* Title of this page */
   const char *zFile;          /* One of "css", "footer", "header", "details" */
   int iSkin;                  /* draft number.  1..9 */
@@ -730,7 +811,7 @@ void setup_skinedit(void){
   if( iSkin<1 || iSkin>9 ) iSkin = 1;
 
   /* Check that the user is authorized to edit this skin. */
-  if( !g.perm.Setup ){
+  if( !g.perm.Admin ){
     char *zAllowedEditors = "";
     Glob *pAllowedEditors;
     int isMatch = 0;
@@ -753,11 +834,10 @@ void setup_skinedit(void){
   if( ii<0 || ii>count(aSkinAttr) ) ii = 0;
   zFile = aSkinAttr[ii].zFile;
   zDraft = mprintf("draft%d", iSkin);
-  zKey = mprintf("draft%d-%s", iSkin, zFile);
   zTitle = mprintf("%s for Draft%d", aSkinAttr[ii].zTitle, iSkin);
   zBasis = PD("basis","current");
   zDflt = skin_file_content(zBasis, zFile);
-  zOrig = db_get(zKey, zDflt);
+  zOrig = db_get_mprintf(zDflt, "draft%d-%s",iSkin,zFile);
   zContent = PD(zFile,zOrig);
   if( P("revert")!=0 && cgi_csrf_safe(0) ){
     zContent = zDflt;
@@ -765,18 +845,19 @@ void setup_skinedit(void){
   }
 
   db_begin_transaction();
+  style_set_current_feature("skins");
   style_header("%s", zTitle);
   for(j=0; j<count(aSkinAttr); j++){
     style_submenu_element(aSkinAttr[j].zSubmenu,
           "%R/setup_skinedit?w=%d&basis=%h&sk=%d",j,zBasis,iSkin);
   }
-  @ <form action="%s(g.zTop)/setup_skinedit" method="post"><div>
+  @ <form action="%R/setup_skinedit" method="post"><div>
   login_insert_csrf_secret();
   @ <input type='hidden' name='w' value='%d(ii)'>
   @ <input type='hidden' name='sk' value='%d(iSkin)'>
   @ <h2>Edit %s(zTitle):</h2>
-  if( P("submit") && cgi_csrf_safe(0) && strcmp(zOrig,zContent)!=0 ){
-    db_set(zKey, zContent, 0);
+  if( P("submit") && cgi_csrf_safe(0) && (zOrig==0 || strcmp(zOrig,zContent)!=0) ){
+    db_set_mprintf(zContent, 0, "draft%d-%s",iSkin,zFile);
   }
   @ <textarea name="%s(zFile)" rows="10" cols="80">\
   @ %h(zContent)</textarea>
@@ -793,18 +874,21 @@ void setup_skinedit(void){
   @ <input type="submit" name="diff" value="Unified Diff" />
   @ <input type="submit" name="sbsdiff" value="Side-by-Side Diff" />
   if( P("diff")!=0 || P("sbsdiff")!=0 ){
-    u64 diffFlags = construct_diff_flags(1) | DIFF_STRIP_EOLCR;
     Blob from, to, out;
-    if( P("sbsdiff")!=0 ) diffFlags |= DIFF_SIDEBYSIDE;
+    DiffConfig DCfg;
+    construct_diff_flags(1, &DCfg);
+    DCfg.diffFlags |= DIFF_STRIP_EOLCR;
+    if( P("sbsdiff")!=0 ) DCfg.diffFlags |= DIFF_SIDEBYSIDE;
     blob_init(&to, zContent, -1);
     blob_init(&from, skin_file_content(zBasis, zFile), -1);
     blob_zero(&out);
-    if( diffFlags & DIFF_SIDEBYSIDE ){
-      text_diff(&from, &to, &out, 0, diffFlags | DIFF_HTML | DIFF_NOTTOOBIG);
+    DCfg.diffFlags |= DIFF_HTML | DIFF_NOTTOOBIG; 
+    if( DCfg.diffFlags & DIFF_SIDEBYSIDE ){
+      text_diff(&from, &to, &out, &DCfg);
       @ %s(blob_str(&out))
     }else{
-      text_diff(&from, &to, &out, 0,
-             diffFlags | DIFF_LINENO | DIFF_HTML | DIFF_NOTTOOBIG);
+      DCfg.diffFlags |= DIFF_LINENO;
+      text_diff(&from, &to, &out, &DCfg);
       @ <pre class="udiff">
       @ %s(blob_str(&out))
       @ </pre>
@@ -814,7 +898,7 @@ void setup_skinedit(void){
     blob_reset(&out);
   }
   @ </div></form>
-  style_footer();
+  style_finish_page();
   db_end_transaction(0);
 }
 
@@ -855,17 +939,19 @@ static void skin_publish(int iSkin){
                        " AND value=%Q", zCurrent);
   }
   if( !seen ){
+    db_unprotect(PROTECT_CONFIG);
     db_multi_exec(
       "INSERT INTO config(name,value,mtime) VALUES("
       "  strftime('skin:Backup On %%Y-%%m-%%d %%H:%%M:%%S'),"
       "  %Q,now())", zCurrent
     );
+    db_protect_pop();
   }
 
   /* Publish draft iSkin */
   for(i=0; i<count(azSkinFile); i++){
     char *zNew = db_get_mprintf("", "draft%d-%s", iSkin, azSkinFile[i]);
-    db_set(azSkinFile[i], zNew, 0);
+    db_set(azSkinFile[i]/*works-like:"x"*/, zNew, 0);
   }
 }
 
@@ -881,7 +967,7 @@ void setup_skin(void){
   int isEditor;   /* Others authorized to make edits */
   char *zAllowedEditors;   /* Who may edit the draft skin */
   char *zBase;             /* Base URL for draft under test */
-  static const char *azTestPages[] = {
+  static const char *const azTestPages[] = {
      "home",
      "timeline",
      "dir?ci=tip",
@@ -903,7 +989,7 @@ void setup_skin(void){
     return;
   }
   zAllowedEditors = db_get_mprintf("", "draft%d-users", iSkin);
-  if( g.perm.Setup ){
+  if( g.perm.Admin ){
     isSetup = isEditor = 1;
   }else{
     Glob *pAllowedEditors;
@@ -929,12 +1015,15 @@ void setup_skin(void){
     skin_publish(iSkin);
   }
 
+  style_set_current_feature("skins");
   style_header("Customize Skin");
 
   @ <p>Customize the look of this Fossil repository by making changes
   @ to the CSS, Header, Footer, and Detail Settings in one of nine "draft"
   @ configurations.  Then, after verifying that all is working correctly,
-  @ publish the draft to become the new main Skin.<p>
+  @ publish the draft to become the new main Skin. Users can select a skin
+  @ of their choice from the built-in ones or the locally-edited one via
+  @ <a href='%R/skins'>the /skins page</a>.</p>
   @
   @ <a name='step1'></a>
   @ <h1>Step 1: Identify Which Draft To Use</h1>
@@ -1057,12 +1146,12 @@ void setup_skin(void){
   @ <a name='step7'></a>
   @ <h1>Step 7: Publish</h1>
   @
-  if( !g.perm.Setup ){
+  if( !g.perm.Admin ){
     @ <p>Only administrators are allowed to publish draft skins.  Contact
     @ an administrator to get this "draft%d(iSkin)" skin published.</p>
   }else{
     @ <p>When the draft%d(iSkin) skin is ready for production use,
-    @ make it the default scan by clicking the acknowledgements and
+    @ make it the default skin by clicking the acknowledgements and
     @ pressing the button below:</p>
     @
     @ <form method='POST' action='%R/setup_skin#step7'>
@@ -1082,13 +1171,70 @@ void setup_skin(void){
   @ <a name='step8'></a>
   @ <h1>Step 8: Cleanup and Undo Actions</h1>
   @
-  if( !g.perm.Setup ){
+  if( !g.perm.Admin ){
     @ <p>Administrators can optionally save or restore legacy skins, and/or
     @ undo a prior publish.
   }else{
     @ <p>Visit the <a href='%R/setup_skin_admin'>Skin Admin</a> page
     @ for cleanup and recovery actions.
   }
-  style_load_one_js_file("skin.js");
-  style_footer();
+  builtin_request_js("skin.js");
+  style_finish_page();
+}
+
+/*
+** WEBPAGE: skins
+**
+** Show a list of all of the built-in skins, plus the responsitory skin,
+** and provide the user with an opportunity to change to any of them.
+*/
+void skins_page(void){
+  int i;
+  char *zBase = fossil_strdup(g.zTop);
+  size_t nBase = strlen(zBase);
+  if( iDraftSkin && sqlite3_strglob("*/draft?", zBase)==0 ){
+    nBase -= 7;
+    zBase[nBase] = 0;
+  }else if( pAltSkin ){
+    char *zPattern = mprintf("*/skn_%s", pAltSkin->zLabel);
+    if( sqlite3_strglob(zPattern, zBase)==0 ){
+      nBase -= strlen(zPattern)-1;
+      zBase[nBase] = 0;
+    }
+    fossil_free(zPattern);
+  } 
+  login_check_credentials();
+  style_header("Skins");
+  if( iDraftSkin || nSkinRank<=1 ){
+    @ <p class="warning">Warning:
+    if( iDraftSkin>0 ){
+      @ you are using a draft skin,
+    }else{
+      @ this fossil instance was started with a hard-coded skin
+      @ value,
+    }
+    @ which trumps any option selected below. A skin selected
+    @ below will be recorded in your preference cookie
+    @ but will not be used so long as the site has a
+    @ higher-priority skin in place.
+    @ </p>
+  }
+  @ <p>The following skins are available for this repository:</p>
+  @ <ul>
+  if( pAltSkin==0 && zAltSkinDir==0 && iDraftSkin==0 ){
+    @ <li> Standard skin for this repository &larr; <i>Currently in use</i>
+  }else{
+    @ <li> %z(href("%R/skins?skin="))Standard skin for this repository</a>
+  }
+  for(i=0; i<count(aBuiltinSkin); i++){
+    if( pAltSkin==&aBuiltinSkin[i] ){
+      @ <li> %h(aBuiltinSkin[i].zDesc) &larr; <i>Currently in use</i>
+    }else{
+      char *zUrl = href("%R/skins?skin=%T", aBuiltinSkin[i].zLabel);
+      @ <li> %z(zUrl)%h(aBuiltinSkin[i].zDesc)</a>
+    }
+  }
+  @ </ul>
+  style_finish_page();
+  fossil_free(zBase);
 }
