@@ -41,39 +41,40 @@
 #define URL_REMEMBER_PW      0x008  /* Should remember pw */
 #define URL_PROMPTED         0x010  /* Prompted for PW already */
 #define URL_OMIT_USER        0x020  /* Omit the user name from URL */
+#define URL_USE_CONFIG       0x040  /* Use remembered URLs from CONFIG table */
+#define URL_USE_PARENT       0x080  /* Use the URL of the parent project */
 
 /*
 ** The URL related data used with this subsystem.
 */
 struct UrlData {
-  int isFile;      /* True if a "file:" url */
-  int isHttps;     /* True if a "https:" url */
-  int isSsh;       /* True if an "ssh:" url */
-  int isAlias;     /* Input URL was an alias */
-  char *name;      /* Hostname for http: or filename for file: */
-  char *hostname;  /* The HOST: parameter on http headers */
+  int isFile;           /* True if a "file:" url */
+  int isHttps;          /* True if a "https:" url */
+  int isSsh;            /* True if an "ssh:" url */
+  int isAlias;          /* Input URL was an alias */
+  char *name;           /* Hostname for http: or filename for file: */
+  char *hostname;       /* The HOST: parameter on http headers */
   const char *protocol; /* "http" or "https" or "ssh" or "file" */
-  int port;        /* TCP port number for http: or https: */
-  int dfltPort;    /* The default port for the given protocol */
-  char *path;      /* Pathname for http: */
-  char *user;      /* User id for http: */
-  char *passwd;    /* Password for http: */
-  char *canonical; /* Canonical representation of the URL */
-  char *proxyAuth; /* Proxy-Authorizer: string */
-  char *fossil;    /* The fossil query parameter on ssh: */
-  unsigned flags;  /* Boolean flags controlling URL processing */
-  int useProxy;    /* Used to remember that a proxy is in use */
-  char *proxyUrlPath;
-  int proxyOrigPort; /* Tunneled port number for https through proxy */
+  int port;             /* TCP port number for http: or https: */
+  int dfltPort;         /* The default port for the given protocol */
+  char *path;           /* Pathname for http: */
+  char *user;           /* User id for http: */
+  char *passwd;         /* Password for http: */
+  char *canonical;      /* Canonical representation of the URL */
+  char *proxyAuth;      /* Proxy-Authorizer: string */
+  char *fossil;         /* The fossil query parameter on ssh: */
+  unsigned flags;       /* Boolean flags controlling URL processing */
+  int useProxy;         /* Used to remember that a proxy is in use */
+  char *proxyUrlPath;   /* Remember path when proxy is use */
+  char *proxyUrlCanonical; /* Remember canonical path when proxy is use */
+  int proxyOrigPort;    /* Tunneled port number for https through proxy */
 };
 #endif /* INTERFACE */
 
 
 /*
-** Parse the given URL.  Or if zUrl is NULL, parse the URL in the
-** last-sync-url setting using last-sync-pw as the password.  Store
-** the parser results in the pUrlData object.  Populate members of pUrlData
-** as follows:
+** Parse the URL in the zUrl argument. Store results in the pUrlData object.
+** Populate members of pUrlData as follows:
 **
 **      isFile      True if FILE:
 **      isHttps     True if HTTPS:
@@ -88,6 +89,16 @@ struct UrlData {
 **      hostname    HOST:PORT or just HOST if port is the default.
 **      canonical   The URL in canonical form, omitting the password
 **
+** If zUrl==0 and URL_USE_CONFIG is set, then parse the URL stored
+** in last-sync-url and last-sync-pw of the CONFIG table.  Or if 
+** URL_USE_PARENT is also set, then use parent-project-url and
+** parent-project-pw from the CONFIG table instead of last-sync-url
+** and last-sync-pw.
+**
+** If zUrl is a symbolic name and URL_USE_CONFIG is true, then look up
+** the URL in sync-url:%Q and sync-pw:%Q elements of the CONFIG table where
+** %Q is the symbolic name.
+**
 ** This routine differs from url_parse() in that this routine stores the
 ** results in pUrlData and does not change the values of global variables.
 ** The url_parse() routine puts its result in g.url.
@@ -100,27 +111,41 @@ void url_parse_local(
   int i, j, c;
   char *zFile = 0;
 
-  if( zUrl==0 || strcmp(zUrl,"default")==0 ){
-    zUrl = db_get("last-sync-url", 0);
-    if( zUrl==0 ) return;
-    if( pUrlData->passwd==0 ){
-      pUrlData->passwd = unobscure(db_get("last-sync-pw", 0));
-    }
-    pUrlData->isAlias = 1;
-  }else{
-    char *zKey = sqlite3_mprintf("sync-url:%q", zUrl);
-    char *zAlt = db_get(zKey, 0);
-    sqlite3_free(zKey);
-    if( zAlt ){
-      pUrlData->passwd = unobscure(
-        db_text(0, "SELECT value FROM config WHERE name='sync-pw:%q'",zUrl)
-      );
-      zUrl = zAlt;
-      urlFlags |= URL_REMEMBER_PW;
+  if( urlFlags & URL_USE_CONFIG ){
+    if( zUrl==0 || strcmp(zUrl,"default")==0 ){
+      const char *zPwConfig = "last-sync-pw";
+      if( urlFlags & URL_USE_PARENT ){
+        zUrl = db_get("parent-project-url", 0);
+        if( zUrl==0 ){
+          zUrl = db_get("last-sync-url",0);
+        }else{
+          zPwConfig = "parent-project-pw";
+        }
+      }else{
+        zUrl = db_get("last-sync-url", 0);
+      }
+      if( zUrl==0 ) return;
+      if( pUrlData->passwd==0 ){
+        pUrlData->passwd = unobscure(db_get(zPwConfig, 0));
+      }
       pUrlData->isAlias = 1;
     }else{
-      pUrlData->isAlias = 0;
+      char *zKey = sqlite3_mprintf("sync-url:%q", zUrl);
+      char *zAlt = db_get(zKey, 0);
+      sqlite3_free(zKey);
+      if( zAlt ){
+        pUrlData->passwd = unobscure(
+          db_text(0, "SELECT value FROM config WHERE name='sync-pw:%q'",zUrl)
+        );
+        zUrl = zAlt;
+        urlFlags |= URL_REMEMBER_PW;
+        pUrlData->isAlias = 1;
+      }else{
+        pUrlData->isAlias = 0;
+      }
     }
+  }else{
+    if( zUrl==0 ) return;
   }
 
   if( strncmp(zUrl, "http://", 7)==0
@@ -143,7 +168,7 @@ void url_parse_local(
       pUrlData->isSsh = 1;
       pUrlData->protocol = "ssh";
       pUrlData->dfltPort = 22;
-      pUrlData->fossil = "fossil";
+      pUrlData->fossil = fossil_strdup("fossil");
       iStart = 6;
     }else{
       pUrlData->isHttps = 0;
@@ -228,7 +253,7 @@ void url_parse_local(
         i++;
       }
       if( fossil_strcmp(zName,"fossil")==0 ){
-        pUrlData->fossil = zValue;
+        pUrlData->fossil = fossil_strdup(zValue);
         dehttpize(pUrlData->fossil);
         fossil_free(zExe);
         zExe = mprintf("%cfossil=%T", cQuerySep, pUrlData->fossil);
@@ -249,7 +274,11 @@ void url_parse_local(
         pUrlData->path, zExe
       );
     }
-    if( pUrlData->isSsh && pUrlData->path[1] ) pUrlData->path++;
+    if( pUrlData->isSsh && pUrlData->path[1] ){
+      char *zOld = pUrlData->path;
+      pUrlData->path = mprintf("%s", zOld+1);
+      fossil_free(zOld);
+    }
     free(zLogin);
   }else if( strncmp(zUrl, "file:", 5)==0 ){
     pUrlData->isFile = 1;
@@ -282,7 +311,7 @@ void url_parse_local(
     free(zFile);
     zFile = 0;
     pUrlData->protocol = "file";
-    pUrlData->path = "";
+    pUrlData->path = mprintf("");
     pUrlData->name = mprintf("%b", &cfile);
     pUrlData->canonical = mprintf("file://%T", pUrlData->name);
     blob_reset(&cfile);
@@ -300,6 +329,70 @@ void url_parse_local(
   }
 }
 
+/*
+** Construct the complete URL for a UrlData object, including the
+** login name and password, into memory obtained from fossil_malloc()
+** and return a pointer to that URL text.
+*/
+char *url_full(const UrlData *p){
+  Blob x = BLOB_INITIALIZER;
+  if( p->isFile || p->user==0 || p->user[0]==0 ){
+    return fossil_strdup(p->canonical);
+  }
+  blob_appendf(&x, "%s://", p->protocol);
+  if( p->user && p->user[0] ){
+    blob_appendf(&x, "%t", p->user);
+    if( p->passwd && p->passwd[0] ){
+      blob_appendf(&x, ":%t", p->passwd);
+    }
+    blob_appendf(&x, "@");
+  }
+  blob_appendf(&x, "%T", p->name);
+  if( p->dfltPort!=p->port ){
+    blob_appendf(&x, ":%d", p->port);
+  }
+  blob_appendf(&x, "%T", p->path);
+  (void)blob_str(&x);
+  return x.aData;
+}
+
+/*
+** Construct a URL for a UrlData object that omits the
+** login name and password, into memory obtained from fossil_malloc()
+** and return a pointer to that URL text.
+*/
+char *url_nouser(const UrlData *p){
+  Blob x = BLOB_INITIALIZER;
+  if( p->isFile || p->user==0 || p->user[0]==0 ){
+    return fossil_strdup(p->canonical);
+  }
+  blob_appendf(&x, "%s://", p->protocol);
+  blob_appendf(&x, "%T", p->name);
+  if( p->dfltPort!=p->port ){
+    blob_appendf(&x, ":%d", p->port);
+  }
+  blob_appendf(&x, "%T", p->path);
+  (void)blob_str(&x);
+  return x.aData;
+}
+
+/*
+** SQL function to remove the username/password from a URL
+*/
+void url_nouser_func(
+  sqlite3_context *context,
+  int argc,
+  sqlite3_value **argv
+){
+  const char *zOrig = (const char*)sqlite3_value_text(argv[0]);
+  UrlData x;
+  if( zOrig==0 ) return;
+  memset(&x, 0, sizeof(x));
+  url_parse_local(zOrig, URL_OMIT_USER, &x);
+  sqlite3_result_text(context, x.canonical, -1, SQLITE_TRANSIENT);
+  url_unparse(&x);
+}
+
 /*
 ** Reclaim malloced memory from a UrlData object
 */
@@ -312,6 +405,7 @@ void url_unparse(UrlData *p){
   fossil_free(p->path);
   fossil_free(p->user);
   fossil_free(p->passwd);
+  fossil_free(p->fossil);
   memset(p, 0, sizeof(*p));
 }
 
@@ -383,6 +477,7 @@ void cmd_test_urlparser(void){
     fossil_print("g.url.canonical = %s\n", g.url.canonical);
     fossil_print("g.url.fossil    = %s\n", g.url.fossil);
     fossil_print("g.url.flags     = 0x%02x\n", g.url.flags);
+    fossil_print("url_full(g.url) = %z\n", url_full(&g.url));
     if( g.url.isFile || g.url.isSsh ) break;
     if( i==0 ){
       fossil_print("********\n");
@@ -442,7 +537,7 @@ void url_enable_proxy(const char *zMsg){
   zProxy = zProxyOpt;
   if( zProxy==0 ){
     zProxy = db_get("proxy", 0);
-    if( zProxy==0 || zProxy[0]==0 || is_false(zProxy) ){
+    if( fossil_strcmp(zProxy, "system")==0 ){
       zProxy = fossil_getenv("http_proxy");
     }
   }
@@ -472,6 +567,7 @@ void url_enable_proxy(const char *zMsg){
     g.url.passwd = zOriginalPasswd;
     g.url.isHttps = fOriginalIsHttps;
     g.url.useProxy = 1;
+    g.url.proxyUrlCanonical = zOriginalUrl;;
     g.url.proxyUrlPath = zOriginalUrlPath;
     g.url.proxyOrigPort = iOriginalPort;
     g.url.flags = uOriginalFlags;
@@ -601,7 +697,7 @@ void url_prompt_for_password_local(UrlData *pUrlData){
    && (pUrlData->flags & URL_PROMPTED)==0
   ){
     pUrlData->flags |= URL_PROMPTED;
-    pUrlData->passwd = prompt_for_user_password(pUrlData->user);
+    pUrlData->passwd = prompt_for_user_password(pUrlData->canonical);
     if( pUrlData->passwd[0]
      && (pUrlData->flags & (URL_REMEMBER|URL_ASK_REMEMBER_PW))!=0
     ){
@@ -630,9 +726,23 @@ void url_prompt_for_password(void){
 */
 void url_remember(void){
   if( g.url.flags & URL_REMEMBER ){
-    db_set("last-sync-url", g.url.canonical, 0);
+    const char *url;
+    if( g.url.useProxy ){
+      url = g.url.proxyUrlCanonical;
+    }else{
+      url = g.url.canonical;
+    }
+    if( g.url.flags & URL_USE_PARENT ){
+      db_set("parent-project-url", url, 0);
+    }else{
+      db_set("last-sync-url", url, 0);
+    }
     if( g.url.user!=0 && g.url.passwd!=0 && ( g.url.flags & URL_REMEMBER_PW ) ){
-      db_set("last-sync-pw", obscure(g.url.passwd), 0);
+      if( g.url.flags & URL_USE_PARENT ){
+        db_set("parent-project-pw", obscure(g.url.passwd), 0);
+      }else{
+        db_set("last-sync-pw", obscure(g.url.passwd), 0);
+      }
     }
   }
 }
